#!/usr/bin/env python3
# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Definition of targets to build artifacts."""

import os.path
import random
import string
import sys

sys.path.insert(0, os.path.abspath(".."))
import python_utils.jobset as jobset

_LATEST_MANYLINUX = "manylinux2014"

REPORT_BASE_PATH = os.getenv("GRPC_TEST_REPORT_BASE_DIR", os.path.abspath("."))


def create_docker_jobspec(
    name,
    dockerfile_dir,
    shell_command,
    environ={},
    flake_retries=0,
    timeout_retries=0,
    timeout_seconds=30 * 60,
    extra_docker_args=None,
    verbose_success=False,
):
    """Creates jobspec for a task running under docker."""
    environ = environ.copy()
    environ["ARTIFACTS_OUT"] = "artifacts/%s" % name

    docker_args = []
    for k, v in list(environ.items()):
        docker_args += ["-e", "%s=%s" % (k, v)]
    docker_env = {
        "DOCKERFILE_DIR": dockerfile_dir,
        "DOCKER_RUN_SCRIPT": "tools/run_tests/dockerize/docker_run.sh",
        "DOCKER_RUN_SCRIPT_COMMAND": shell_command,
        "OUTPUT_DIR": "artifacts",
    }
    if extra_docker_args is not None:
        docker_env["EXTRA_DOCKER_ARGS"] = extra_docker_args
    jobspec = jobset.JobSpec(
        cmdline=["tools/run_tests/dockerize/build_and_run_docker.sh"]
        + docker_args,
        environ=docker_env,
        shortname="build_artifact.%s" % (name),
        timeout_seconds=timeout_seconds,
        flake_retries=flake_retries,
        timeout_retries=timeout_retries,
        verbose_success=verbose_success,
        logfilename=os.path.abspath(
            f"{REPORT_BASE_PATH}/reports/artifact.{name}.log"
        ),
    )
    return jobspec


def create_jobspec(
    name,
    cmdline,
    environ={},
    shell=False,
    flake_retries=0,
    timeout_retries=0,
    timeout_seconds=30 * 60,
    use_workspace=False,
    cpu_cost=1.0,
    verbose_success=False,
):
    """Creates jobspec."""
    environ = environ.copy()
    if use_workspace:
        environ["WORKSPACE_NAME"] = "workspace_%s" % name
        environ["ARTIFACTS_OUT"] = os.path.join("..", "artifacts", name)
        cmdline = [
            "bash",
            "tools/run_tests/artifacts/run_in_workspace.sh",
        ] + cmdline
    else:
        environ["ARTIFACTS_OUT"] = os.path.join("artifacts", name)

    jobspec = jobset.JobSpec(
        cmdline=cmdline,
        environ=environ,
        shortname="build_artifact.%s" % (name),
        timeout_seconds=timeout_seconds,
        flake_retries=flake_retries,
        timeout_retries=timeout_retries,
        shell=shell,
        cpu_cost=cpu_cost,
        verbose_success=verbose_success,
        logfilename=os.path.abspath(
            f"{REPORT_BASE_PATH}/reports/artifact.{name}.log"
        ),
    )
    return jobspec


_MACOS_COMPAT_FLAG = "-mmacosx-version-min=10.10"

_ARCH_FLAG_MAP = {"x86": "-m32", "x64": "-m64"}


class PythonArtifact:
    """Builds Python artifacts."""

    def __init__(self, platform, arch, py_version, presubmit=False):
        self.name = "python_%s_%s_%s" % (platform, arch, py_version)
        self.platform = platform
        self.arch = arch
        self.labels = ["artifact", "python", platform, arch, py_version]
        if presubmit:
            self.labels.append("presubmit")
        self.py_version = py_version
        if platform == _LATEST_MANYLINUX:
            self.labels.append("latest-manylinux")
        if "manylinux" in platform:
            self.labels.append("linux")
        if "linux_extra" in platform:
            # linux_extra wheels used to be built by a separate kokoro job.
            # Their build is now much faster, so they can be included
            # in the regular artifact build.
            self.labels.append("linux")
        if "musllinux" in platform:
            self.labels.append("linux")

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        environ = {}
        if inner_jobs is not None:
            # set number of parallel jobs when building native extension
            # building the native extension is the most time-consuming part of the build
            environ["GRPC_PYTHON_BUILD_EXT_COMPILER_JOBS"] = str(inner_jobs)

        if self.platform == "macos":
            environ["ARCHFLAGS"] = "-arch arm64 -arch x86_64"
            environ["GRPC_BUILD_MAC"] = "true"

        if self.platform == "linux_extra":
            # Crosscompilation build for armv7 (e.g. Raspberry Pi)
            environ["PYTHON"] = "/opt/python/{}/bin/python3".format(
                self.py_version
            )
            environ["PIP"] = "/opt/python/{}/bin/pip3".format(self.py_version)
            environ["GRPC_SKIP_PIP_CYTHON_UPGRADE"] = "TRUE"
            environ["GRPC_SKIP_TWINE_CHECK"] = "TRUE"
            environ["LDFLAGS"] = "-s"
            return create_docker_jobspec(
                self.name,
                "tools/dockerfile/grpc_artifact_python_linux_{}".format(
                    self.arch
                ),
                "tools/run_tests/artifacts/build_artifact_python.sh",
                environ=environ,
                timeout_seconds=60 * 60 * 4,
            )
        elif "manylinux" in self.platform:
            if self.arch == "x86":
                environ["SETARCH_CMD"] = "linux32"
                environ["GRPC_SKIP_TWINE_CHECK"] = "TRUE"
            # Inside the manylinux container, the python installations are located in
            # special places...
            environ["PYTHON"] = "/opt/python/{}/bin/python".format(
                self.py_version
            )
            environ["PIP"] = "/opt/python/{}/bin/pip".format(self.py_version)
            environ["GRPC_SKIP_PIP_CYTHON_UPGRADE"] = "TRUE"

            # currently no manylinux architectures (including aarch64)
            # require cross-compiling, hence the below values are set for all
            # manylinux targets

            environ["GRPC_RUN_AUDITWHEEL_REPAIR"] = "TRUE"
            # only build the packages that depend on grpcio-tools
            # if we're not crosscompiling.
            # - they require protoc to run on current architecture
            # - they only have sdist packages anyway, so it's useless to build them again
            environ["GRPC_BUILD_GRPCIO_TOOLS_DEPENDENTS"] = "TRUE"

            return create_docker_jobspec(
                self.name,
                "tools/dockerfile/grpc_artifact_python_%s_%s"
                % (self.platform, self.arch),
                "tools/run_tests/artifacts/build_artifact_python.sh",
                environ=environ,
                timeout_seconds=60 * 60 * 4,
            )
        elif "musllinux" in self.platform:
            environ["PYTHON"] = "/opt/python/{}/bin/python".format(
                self.py_version
            )
            environ["PIP"] = "/opt/python/{}/bin/pip".format(self.py_version)
            environ["GRPC_SKIP_PIP_CYTHON_UPGRADE"] = "TRUE"
            environ["GRPC_PYTHON_BUILD_WITH_STATIC_LIBSTDCXX"] = "TRUE"

            if self.arch in ("x86"):
                environ["GRPC_SKIP_TWINE_CHECK"] = "TRUE"

            # We're using musllinux aarch64 image to build this artifact so no crosscompiling required.
            environ["GRPC_BUILD_GRPCIO_TOOLS_DEPENDENTS"] = "TRUE"
            environ["GRPC_RUN_AUDITWHEEL_REPAIR"] = "TRUE"

            return create_docker_jobspec(
                self.name,
                "tools/dockerfile/grpc_artifact_python_%s_%s"
                % (self.platform, self.arch),
                "tools/run_tests/artifacts/build_artifact_python.sh",
                environ=environ,
                timeout_seconds=60 * 60 * 4,
            )
        elif self.platform == "windows":
            environ["EXT_COMPILER"] = "msvc"
            # For some reason, the batch script %random% always runs with the same
            # seed.  We create a random temp-dir here
            dir = "".join(
                random.choice(string.ascii_uppercase) for _ in range(10)
            )
            return create_jobspec(
                self.name,
                [
                    "tools\\run_tests\\artifacts\\build_artifact_python.bat",
                    self.py_version,
                    "32" if self.arch == "x86" else "64",
                ],
                environ=environ,
                timeout_seconds=60 * 60 * 2,
                use_workspace=True,
            )
        else:
            environ["PYTHON"] = self.py_version
            environ["SKIP_PIP_INSTALL"] = "TRUE"
            return create_jobspec(
                self.name,
                ["tools/run_tests/artifacts/build_artifact_python.sh"],
                environ=environ,
                timeout_seconds=60 * 60 * 4,
                use_workspace=True,
            )

    def __str__(self):
        return self.name


class RubyArtifact:
    """Builds ruby native gem."""

    def __init__(self, platform, gem_platform, presubmit=False):
        self.name = "ruby_native_gem_%s_%s" % (platform, gem_platform)
        self.platform = platform
        self.gem_platform = gem_platform
        platform_label = self.platform
        self.labels = ["artifact", "ruby", platform, gem_platform]
        if presubmit:
            self.labels.append("presubmit")

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        environ = {}
        if inner_jobs is not None:
            # set number of parallel jobs when building native extension
            environ["GRPC_RUBY_BUILD_PROCS"] = str(inner_jobs)

        # determine build_type at runtime.
        # By default, build all supported ruby versions ("continuous").
        # If explicitly running in presubmit, build only a subset of versions to save time.
        build_type = "continuous"
        if os.environ.get("KOKORO_JOB_TYPE") == "PRESUBMIT":
            build_type = "presubmit"

        # Ruby build uses docker internally and docker cannot be nested.
        # We are using a custom workspace instead.
        return create_jobspec(
            self.name,
            [
                "tools/run_tests/artifacts/build_artifact_ruby.sh",
                self.gem_platform,
                build_type,
            ],
            use_workspace=True,
            timeout_seconds=240 * 60,
            environ=environ,
        )


class PHPArtifact:
    """Builds PHP PECL package"""

    def __init__(self, platform, arch, presubmit=False):
        self.name = "php_pecl_package_{0}_{1}".format(platform, arch)
        self.platform = platform
        self.arch = arch
        self.labels = ["artifact", "php", platform, arch]
        if presubmit:
            self.labels.append("presubmit")

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        del inner_jobs  # arg unused as PHP artifact build is basically just packing an archive
        if self.platform == "linux":
            return create_docker_jobspec(
                self.name,
                "tools/dockerfile/test/php8_zts_debian12_{}".format(self.arch),
                "tools/run_tests/artifacts/build_artifact_php.sh",
            )
        else:
            return create_jobspec(
                self.name,
                ["tools/run_tests/artifacts/build_artifact_php.sh"],
                use_workspace=True,
            )


class ProtocArtifact:
    """Builds protoc and protoc-plugin artifacts"""

    def __init__(self, platform, arch, presubmit=False):
        self.name = "protoc_%s_%s" % (platform, arch)
        self.platform = platform
        self.arch = arch
        self.labels = ["artifact", "protoc", platform, arch]
        if presubmit:
            self.labels.append("presubmit")

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        environ = {}
        if inner_jobs is not None:
            # set number of parallel jobs when building protoc
            environ["GRPC_PROTOC_BUILD_COMPILER_JOBS"] = str(inner_jobs)

        if self.platform != "windows":
            environ["CXXFLAGS"] = ""
            environ["LDFLAGS"] = ""
            if self.platform == "linux":
                dockerfile_dir = (
                    "tools/dockerfile/grpc_artifact_manylinux2014_{}".format(
                        self.arch
                    )
                )
                if self.arch == "aarch64":
                    # for aarch64, use a dockcross manylinux image that will
                    # give us both ready to use crosscompiler and sufficient backward compatibility
                    dockerfile_dir = (
                        "tools/dockerfile/grpc_artifact_manylinux2014_aarch64"
                    )
                    # The manylinux2014 cross-compilation toolchain may produce
                    # binaries with 4KB page alignment, but ARM64 Linux kernels
                    # (e.g. RHEL) can use 64KB pages. Without this flag the
                    # protoc binary will segfault (exit code 139) on such systems.
                    # See https://github.com/grpc/grpc/issues/38538
                    # and https://github.com/pypa/manylinux/issues/735.
                    environ["LDFLAGS"] += " -Wl,-z,max-page-size=65536"
                environ["LDFLAGS"] += " -static-libgcc -static-libstdc++ -s"
                return create_docker_jobspec(
                    self.name,
                    dockerfile_dir,
                    "tools/run_tests/artifacts/build_artifact_protoc.sh",
                    environ=environ,
                )
            else:
                environ["CXXFLAGS"] += (
                    " -std=c++17 -stdlib=libc++ %s" % _MACOS_COMPAT_FLAG
                )
                return create_jobspec(
                    self.name,
                    ["tools/run_tests/artifacts/build_artifact_protoc.sh"],
                    environ=environ,
                    timeout_seconds=60 * 60,
                    use_workspace=True,
                )
        else:
            vs_tools_architecture = (
                self.arch
            )  # architecture selector passed to vcvarsall.bat
            environ["ARCHITECTURE"] = vs_tools_architecture
            return create_jobspec(
                self.name,
                ["tools\\run_tests\\artifacts\\build_artifact_protoc.bat"],
                environ=environ,
                use_workspace=True,
            )

    def __str__(self):
        return self.name


def _reorder_targets_for_build_speed(targets):
    """Reorder targets to achieve optimal build speed"""
    # ruby artifact build builds multiple artifacts at once, so make sure
    # we start building ruby artifacts first, so that they don't end up
    # being a long tail once everything else finishes.
    return list(
        sorted(
            targets,
            key=lambda target: 0 if target.name.startswith("ruby_") else 1,
        )
    )


def targets():
    """Gets list of supported targets"""
    return _reorder_targets_for_build_speed(
        [
            ProtocArtifact("linux", "x64", presubmit=True),
            ProtocArtifact("linux", "x86", presubmit=True),
            ProtocArtifact("linux", "aarch64", presubmit=True),
            ProtocArtifact("macos", "x64", presubmit=True),
            ProtocArtifact("windows", "x64", presubmit=True),
            ProtocArtifact("windows", "x86", presubmit=True),
            PythonArtifact(
                "manylinux2014", "x64", "cp310-cp310", presubmit=True
            ),
            PythonArtifact("manylinux2014", "x64", "cp311-cp311"),
            PythonArtifact("manylinux2014", "x64", "cp312-cp312"),
            PythonArtifact("manylinux2014", "x64", "cp313-cp313"),
            PythonArtifact(
                "manylinux2014", "x64", "cp314-cp314", presubmit=True
            ),
            PythonArtifact(
                "manylinux2014", "x86", "cp310-cp310", presubmit=True
            ),
            PythonArtifact("manylinux2014", "x86", "cp311-cp311"),
            PythonArtifact("manylinux2014", "x86", "cp312-cp312"),
            PythonArtifact("manylinux2014", "x86", "cp313-cp313"),
            PythonArtifact(
                "manylinux2014", "x86", "cp314-cp314", presubmit=True
            ),
            PythonArtifact(
                "manylinux2014", "aarch64", "cp310-cp310", presubmit=True
            ),
            PythonArtifact("manylinux2014", "aarch64", "cp311-cp311"),
            PythonArtifact("manylinux2014", "aarch64", "cp312-cp312"),
            PythonArtifact("manylinux2014", "aarch64", "cp313-cp313"),
            PythonArtifact(
                "manylinux2014", "aarch64", "cp314-cp314", presubmit=True
            ),
            PythonArtifact(
                "linux_extra", "armv7", "cp310-cp310", presubmit=True
            ),
            PythonArtifact("linux_extra", "armv7", "cp311-cp311"),
            PythonArtifact("linux_extra", "armv7", "cp312-cp312"),
            PythonArtifact("linux_extra", "armv7", "cp313-cp313"),
            PythonArtifact(
                "linux_extra", "armv7", "cp314-cp314", presubmit=True
            ),
            PythonArtifact(
                "musllinux_1_2", "x64", "cp310-cp310", presubmit=True
            ),
            PythonArtifact("musllinux_1_2", "x64", "cp311-cp311"),
            PythonArtifact("musllinux_1_2", "x64", "cp312-cp312"),
            PythonArtifact("musllinux_1_2", "x64", "cp313-cp313"),
            PythonArtifact(
                "musllinux_1_2", "x64", "cp314-cp314", presubmit=True
            ),
            PythonArtifact(
                "musllinux_1_2", "x86", "cp310-cp310", presubmit=True
            ),
            PythonArtifact("musllinux_1_2", "x86", "cp311-cp311"),
            PythonArtifact("musllinux_1_2", "x86", "cp312-cp312"),
            PythonArtifact("musllinux_1_2", "x86", "cp313-cp313"),
            PythonArtifact(
                "musllinux_1_2", "x86", "cp314-cp314", presubmit=True
            ),
            PythonArtifact(
                "musllinux_1_2", "aarch64", "cp310-cp310", presubmit=True
            ),
            PythonArtifact("musllinux_1_2", "aarch64", "cp311-cp311"),
            PythonArtifact("musllinux_1_2", "aarch64", "cp312-cp312"),
            PythonArtifact("musllinux_1_2", "aarch64", "cp313-cp313"),
            PythonArtifact(
                "musllinux_1_2", "aarch64", "cp314-cp314", presubmit=True
            ),
            PythonArtifact("macos", "x64", "python3.10", presubmit=True),
            PythonArtifact("macos", "x64", "python3.11"),
            PythonArtifact("macos", "x64", "python3.12"),
            PythonArtifact("macos", "x64", "python3.13"),
            PythonArtifact("macos", "x64", "python3.14", presubmit=True),
            PythonArtifact("windows", "x86", "Python310_32bit", presubmit=True),
            PythonArtifact("windows", "x86", "Python311_32bit"),
            PythonArtifact("windows", "x86", "Python312_32bit"),
            PythonArtifact("windows", "x86", "Python313_32bit"),
            PythonArtifact("windows", "x86", "Python314_32bit", presubmit=True),
            PythonArtifact("windows", "x64", "Python310", presubmit=True),
            PythonArtifact("windows", "x64", "Python311"),
            PythonArtifact("windows", "x64", "Python312"),
            PythonArtifact("windows", "x64", "Python313"),
            PythonArtifact("windows", "x64", "Python314", presubmit=True),
            RubyArtifact("linux", "x86-mingw32", presubmit=True),
            RubyArtifact("linux", "x64-mingw-ucrt"),
            RubyArtifact("linux", "x86_64-linux-gnu", presubmit=True),
            RubyArtifact("linux", "x86_64-linux-musl", presubmit=True),
            RubyArtifact("linux", "x86-linux-gnu"),
            RubyArtifact("linux", "x86-linux-musl", presubmit=True),
            RubyArtifact("linux", "aarch64-linux-gnu", presubmit=True),
            RubyArtifact("linux", "aarch64-linux-musl"),
            RubyArtifact("linux", "x86_64-darwin"),
            RubyArtifact("linux", "arm64-darwin", presubmit=True),
            PHPArtifact("linux", "x64", presubmit=True),
            PHPArtifact("macos", "x64", presubmit=True),
        ]
    )



# Copyright 2026 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import shutil
import xml.etree.ElementTree as ET


def main():
    # Parse package.xml to find all files to include in the mirror
    tree = ET.parse("package.xml")
    root = tree.getroot()

    # PEAR package.xml namespace
    ns = {"ns": "http://pear.php.net/dtd/package-2.0"}
    files = root.findall(".//ns:file", ns)

    dest_dir = "mirror"
    os.makedirs(dest_dir, exist_ok=True)

    for f in files:
        file_path = f.attrib.get("name")
        if not file_path:
            continue

        src = file_path
        dst = os.path.join(dest_dir, file_path)

        if os.path.exists(src):
            os.makedirs(os.path.dirname(dst), exist_ok=True)
            shutil.copy2(src, dst)
            print(f"Copied: {src} -> {dst}")
        else:
            print(f"Warning: Source file not found: {src}")

    # Copy package.xml itself into the mirror for compatibility/reference
    shutil.copy2("package.xml", os.path.join(dest_dir, "package.xml"))
    print("Copied package.xml to mirror")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/network/tcp_proxy/v3/tcp_proxy.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/network/tcp_proxy/v3/tcp_proxy.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.accesslog.v3 import accesslog_pb2 as envoy_dot_config_dot_accesslog_dot_v3_dot_accesslog__pb2
from envoy.config.core.v3 import backoff_pb2 as envoy_dot_config_dot_core_dot_v3_dot_backoff__pb2
from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.config.core.v3 import config_source_pb2 as envoy_dot_config_dot_core_dot_v3_dot_config__source__pb2
from envoy.config.core.v3 import proxy_protocol_pb2 as envoy_dot_config_dot_core_dot_v3_dot_proxy__protocol__pb2
from envoy.extensions.filters.network.http_connection_manager.v3 import http_connection_manager_pb2 as envoy_dot_extensions_dot_filters_dot_network_dot_http__connection__manager_dot_v3_dot_http__connection__manager__pb2
from envoy.type.v3 import hash_policy_pb2 as envoy_dot_type_dot_v3_dot_hash__policy__pb2
from envoy.type.v3 import percent_pb2 as envoy_dot_type_dot_v3_dot_percent__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from envoy.annotations import deprecation_pb2 as envoy_dot_annotations_dot_deprecation__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n=envoy/extensions/filters/network/tcp_proxy/v3/tcp_proxy.proto\x12-envoy.extensions.filters.network.tcp_proxy.v3\x1a)envoy/config/accesslog/v3/accesslog.proto\x1a\"envoy/config/core/v3/backoff.proto\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a(envoy/config/core/v3/config_source.proto\x1a)envoy/config/core/v3/proxy_protocol.proto\x1aYenvoy/extensions/filters/network/http_connection_manager/v3/http_connection_manager.proto\x1a\x1f\x65nvoy/type/v3/hash_policy.proto\x1a\x1b\x65nvoy/type/v3/percent.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xc4\x15\n\x08TcpProxy\x12\x1c\n\x0bstat_prefix\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x11\n\x07\x63luster\x18\x02 \x01(\tH\x00\x12\x64\n\x11weighted_clusters\x18\n \x01(\x0b\x32G.envoy.extensions.filters.network.tcp_proxy.v3.TcpProxy.WeightedClusterH\x00\x12S\n\ton_demand\x18\x0e \x01(\x0b\x32@.envoy.extensions.filters.network.tcp_proxy.v3.TcpProxy.OnDemand\x12\x36\n\x0emetadata_match\x18\t \x01(\x0b\x32\x1e.envoy.config.core.v3.Metadata\x12/\n\x0cidle_timeout\x18\x08 \x01(\x0b\x32\x19.google.protobuf.Duration\x12:\n\x17\x64ownstream_idle_timeout\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x38\n\x15upstream_idle_timeout\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x38\n\naccess_log\x18\x05 \x03(\x0b\x32$.envoy.config.accesslog.v3.AccessLog\x12\x43\n\x14max_connect_attempts\x18\x07 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x07\xfa\x42\x04*\x02(\x01\x12>\n\x0f\x62\x61\x63koff_options\x18\x12 \x01(\x0b\x32%.envoy.config.core.v3.BackoffStrategy\x12\x38\n\x0bhash_policy\x18\x0b \x03(\x0b\x32\x19.envoy.type.v3.HashPolicyB\x08\xfa\x42\x05\x92\x01\x02\x10\x01\x12\x61\n\x10tunneling_config\x18\x0c \x01(\x0b\x32G.envoy.extensions.filters.network.tcp_proxy.v3.TcpProxy.TunnelingConfig\x12S\n\"max_downstream_connection_duration\x18\r \x01(\x0b\x32\x19.google.protobuf.DurationB\x0c\xfa\x42\t\xaa\x01\x06\x32\x04\x10\xc0\x84=\x12T\n4max_downstream_connection_duration_jitter_percentage\x18\x14 \x01(\x0b\x32\x16.envoy.type.v3.Percent\x12U\n\x19\x61\x63\x63\x65ss_log_flush_interval\x18\x0f \x01(\x0b\x32\x19.google.protobuf.DurationB\x17\x18\x01\xfa\x42\t\xaa\x01\x06\x32\x04\x10\xc0\x84=\x92\xc7\x86\xd8\x04\x03\x33.0\x12\x32\n\x1d\x66lush_access_log_on_connected\x18\x10 \x01(\x08\x42\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12g\n\x12\x61\x63\x63\x65ss_log_options\x18\x11 \x01(\x0b\x32K.envoy.extensions.filters.network.tcp_proxy.v3.TcpProxy.TcpAccessLogOptions\x12;\n\x13proxy_protocol_tlvs\x18\x13 \x03(\x0b\x32\x1e.envoy.config.core.v3.TlvEntry\x12k\n\x15upstream_connect_mode\x18\x15 \x01(\x0e\x32\x42.envoy.extensions.filters.network.tcp_proxy.v3.UpstreamConnectModeB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12\x45\n\x14max_early_data_bytes\x18\x16 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\t\xfa\x42\x06*\x04\x18\x80\x80@\x1a\xa0\x03\n\x0fWeightedCluster\x12q\n\x08\x63lusters\x18\x01 \x03(\x0b\x32U.envoy.extensions.filters.network.tcp_proxy.v3.TcpProxy.WeightedCluster.ClusterWeightB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x1a\xcf\x01\n\rClusterWeight\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x17\n\x06weight\x18\x02 \x01(\rB\x07\xfa\x42\x04*\x02(\x01\x12\x36\n\x0emetadata_match\x18\x03 \x01(\x0b\x32\x1e.envoy.config.core.v3.Metadata:V\x9a\xc5\x88\x1eQ\nOenvoy.config.filter.network.tcp_proxy.v2.TcpProxy.WeightedCluster.ClusterWeight:H\x9a\xc5\x88\x1e\x43\nAenvoy.config.filter.network.tcp_proxy.v2.TcpProxy.WeightedCluster\x1a\xdb\x03\n\x0fTunnelingConfig\x12\x19\n\x08hostname\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x10\n\x08use_post\x18\x02 \x01(\x08\x12J\n\x0eheaders_to_add\x18\x03 \x03(\x0b\x32\'.envoy.config.core.v3.HeaderValueOptionB\t\xfa\x42\x06\x92\x01\x03\x10\xe8\x07\x12\"\n\x1apropagate_response_headers\x18\x04 \x01(\x08\x12\x11\n\tpost_path\x18\x05 \x01(\t\x12#\n\x1bpropagate_response_trailers\x18\x06 \x01(\x08\x12m\n\x14request_id_extension\x18\x07 \x01(\x0b\x32O.envoy.extensions.filters.network.http_connection_manager.v3.RequestIDExtension\x12\x19\n\x11request_id_header\x18\x08 \x01(\t\x12\x1f\n\x17request_id_metadata_key\x18\t \x01(\t:H\x9a\xc5\x88\x1e\x43\nAenvoy.config.filter.network.tcp_proxy.v2.TcpProxy.TunnelingConfig\x1a\x8b\x01\n\x08OnDemand\x12\x38\n\x0codcds_config\x18\x01 \x01(\x0b\x32\".envoy.config.core.v3.ConfigSource\x12\x19\n\x11resources_locator\x18\x02 \x01(\t\x12*\n\x07timeout\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x1a\x88\x01\n\x13TcpAccessLogOptions\x12J\n\x19\x61\x63\x63\x65ss_log_flush_interval\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationB\x0c\xfa\x42\t\xaa\x01\x06\x32\x04\x10\xc0\x84=\x12%\n\x1d\x66lush_access_log_on_connected\x18\x02 \x01(\x08:8\x9a\xc5\x88\x1e\x33\n1envoy.config.filter.network.tcp_proxy.v2.TcpProxyB\x18\n\x11\x63luster_specifier\x12\x03\xf8\x42\x01J\x04\x08\x06\x10\x07R\rdeprecated_v1*]\n\x13UpstreamConnectMode\x12\r\n\tIMMEDIATE\x10\x00\x12\x16\n\x12ON_DOWNSTREAM_DATA\x10\x01\x12\x1f\n\x1bON_DOWNSTREAM_TLS_HANDSHAKE\x10\x02\x42\xb8\x01\n;io.envoyproxy.envoy.extensions.filters.network.tcp_proxy.v3B\rTcpProxyProtoP\x01Z`github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/tcp_proxy/v3;tcp_proxyv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.network.tcp_proxy.v3.tcp_proxy_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n;io.envoyproxy.envoy.extensions.filters.network.tcp_proxy.v3B\rTcpProxyProtoP\001Z`github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/tcp_proxy/v3;tcp_proxyv3\272\200\310\321\006\002\020\002'
  _globals['_TCPPROXY_WEIGHTEDCLUSTER_CLUSTERWEIGHT'].fields_by_name['name']._loaded_options = None
  _globals['_TCPPROXY_WEIGHTEDCLUSTER_CLUSTERWEIGHT'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_TCPPROXY_WEIGHTEDCLUSTER_CLUSTERWEIGHT'].fields_by_name['weight']._loaded_options = None
  _globals['_TCPPROXY_WEIGHTEDCLUSTER_CLUSTERWEIGHT'].fields_by_name['weight']._serialized_options = b'\372B\004*\002(\001'
  _globals['_TCPPROXY_WEIGHTEDCLUSTER_CLUSTERWEIGHT']._loaded_options = None
  _globals['_TCPPROXY_WEIGHTEDCLUSTER_CLUSTERWEIGHT']._serialized_options = b'\232\305\210\036Q\nOenvoy.config.filter.network.tcp_proxy.v2.TcpProxy.WeightedCluster.ClusterWeight'
  _globals['_TCPPROXY_WEIGHTEDCLUSTER'].fields_by_name['clusters']._loaded_options = None
  _globals['_TCPPROXY_WEIGHTEDCLUSTER'].fields_by_name['clusters']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_TCPPROXY_WEIGHTEDCLUSTER']._loaded_options = None
  _globals['_TCPPROXY_WEIGHTEDCLUSTER']._serialized_options = b'\232\305\210\036C\nAenvoy.config.filter.network.tcp_proxy.v2.TcpProxy.WeightedCluster'
  _globals['_TCPPROXY_TUNNELINGCONFIG'].fields_by_name['hostname']._loaded_options = None
  _globals['_TCPPROXY_TUNNELINGCONFIG'].fields_by_name['hostname']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_TCPPROXY_TUNNELINGCONFIG'].fields_by_name['headers_to_add']._loaded_options = None
  _globals['_TCPPROXY_TUNNELINGCONFIG'].fields_by_name['headers_to_add']._serialized_options = b'\372B\006\222\001\003\020\350\007'
  _globals['_TCPPROXY_TUNNELINGCONFIG']._loaded_options = None
  _globals['_TCPPROXY_TUNNELINGCONFIG']._serialized_options = b'\232\305\210\036C\nAenvoy.config.filter.network.tcp_proxy.v2.TcpProxy.TunnelingConfig'
  _globals['_TCPPROXY_TCPACCESSLOGOPTIONS'].fields_by_name['access_log_flush_interval']._loaded_options = None
  _globals['_TCPPROXY_TCPACCESSLOGOPTIONS'].fields_by_name['access_log_flush_interval']._serialized_options = b'\372B\t\252\001\0062\004\020\300\204='
  _globals['_TCPPROXY'].oneofs_by_name['cluster_specifier']._loaded_options = None
  _globals['_TCPPROXY'].oneofs_by_name['cluster_specifier']._serialized_options = b'\370B\001'
  _globals['_TCPPROXY'].fields_by_name['stat_prefix']._loaded_options = None
  _globals['_TCPPROXY'].fields_by_name['stat_prefix']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_TCPPROXY'].fields_by_name['max_connect_attempts']._loaded_options = None
  _globals['_TCPPROXY'].fields_by_name['max_connect_attempts']._serialized_options = b'\372B\004*\002(\001'
  _globals['_TCPPROXY'].fields_by_name['hash_policy']._loaded_options = None
  _globals['_TCPPROXY'].fields_by_name['hash_policy']._serialized_options = b'\372B\005\222\001\002\020\001'
  _globals['_TCPPROXY'].fields_by_name['max_downstream_connection_duration']._loaded_options = None
  _globals['_TCPPROXY'].fields_by_name['max_downstream_connection_duration']._serialized_options = b'\372B\t\252\001\0062\004\020\300\204='
  _globals['_TCPPROXY'].fields_by_name['access_log_flush_interval']._loaded_options = None
  _globals['_TCPPROXY'].fields_by_name['access_log_flush_interval']._serialized_options = b'\030\001\372B\t\252\001\0062\004\020\300\204=\222\307\206\330\004\0033.0'
  _globals['_TCPPROXY'].fields_by_name['flush_access_log_on_connected']._loaded_options = None
  _globals['_TCPPROXY'].fields_by_name['flush_access_log_on_connected']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_TCPPROXY'].fields_by_name['upstream_connect_mode']._loaded_options = None
  _globals['_TCPPROXY'].fields_by_name['upstream_connect_mode']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_TCPPROXY'].fields_by_name['max_early_data_bytes']._loaded_options = None
  _globals['_TCPPROXY'].fields_by_name['max_early_data_bytes']._serialized_options = b'\372B\006*\004\030\200\200@'
  _globals['_TCPPROXY']._loaded_options = None
  _globals['_TCPPROXY']._serialized_options = b'\232\305\210\0363\n1envoy.config.filter.network.tcp_proxy.v2.TcpProxy'
  _globals['_UPSTREAMCONNECTMODE']._serialized_start=3413
  _globals['_UPSTREAMCONNECTMODE']._serialized_end=3506
  _globals['_TCPPROXY']._serialized_start=655
  _globals['_TCPPROXY']._serialized_end=3411
  _globals['_TCPPROXY_WEIGHTEDCLUSTER']._serialized_start=2131
  _globals['_TCPPROXY_WEIGHTEDCLUSTER']._serialized_end=2547
  _globals['_TCPPROXY_WEIGHTEDCLUSTER_CLUSTERWEIGHT']._serialized_start=2266
  _globals['_TCPPROXY_WEIGHTEDCLUSTER_CLUSTERWEIGHT']._serialized_end=2473
  _globals['_TCPPROXY_TUNNELINGCONFIG']._serialized_start=2550
  _globals['_TCPPROXY_TUNNELINGCONFIG']._serialized_end=3025
  _globals['_TCPPROXY_ONDEMAND']._serialized_start=3028
  _globals['_TCPPROXY_ONDEMAND']._serialized_end=3167
  _globals['_TCPPROXY_TCPACCESSLOGOPTIONS']._serialized_start=3170
  _globals['_TCPPROXY_TCPACCESSLOGOPTIONS']._serialized_end=3306
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3

# Copyright 2022 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Generate experiment related code artifacts.

Invoke as: tools/codegen/core/gen_experiments.py
Experiment definitions are in src/core/lib/experiments/experiments.yaml
"""

from __future__ import print_function

import argparse
import os
import sys

import experiments_compiler as exp
import yaml

REPO_ROOT = os.path.normpath(
    os.path.join(os.path.dirname(__file__), "../../..")
)
print(REPO_ROOT)
os.chdir(REPO_ROOT)

DEFAULTS = {
    "broken": "false",
    False: "false",
    True: "true",
    "debug": "kDefaultForDebugOnly",
}

PLATFORMS_DEFINE = {
    "windows": "GPR_WINDOWS",
    "ios": "GRPC_CFSTREAM",
    "posix": "",
}

FINAL_RETURN = {
    "broken": "return false;",
    False: "return false;",
    True: "return true;",
    "debug": "\n#ifdef NDEBUG\nreturn false;\n#else\nreturn true;\n#endif\n",
}

FINAL_DEFINE = {
    "broken": None,
    False: None,
    True: "#define %s",
    "debug": "#ifndef NDEBUG\n#define %s\n#endif",
}

BZL_LIST_FOR_DEFAULTS = {
    "broken": None,
    False: "off",
    True: "on",
    "debug": "dbg",
}


def ParseCommandLineArguments(args):
    """Wrapper for argparse command line arguments handling.

    Args:
    args: List of command line arguments.

    Returns:
    Command line arguments namespace built by argparse.ArgumentParser().
    """
    # formatter_class=argparse.ArgumentDefaultsHelpFormatter is not used here
    # intentionally, We want more formatting than this class can provide.
    flag_parser = argparse.ArgumentParser()
    flag_parser.add_argument(
        "--check",
        action="store_false",
        help="If specified, disables checking experiment expiry dates",
    )
    flag_parser.add_argument(
        "--no_dbg_experiments",
        action="store_true",
        help="Prohibit 'debug' configurations",
        default=False,
    )
    return flag_parser.parse_args(args)


args = ParseCommandLineArguments(sys.argv[1:])


def _InjectGithubPath(path):
    base, ext = os.path.splitext(path)
    return base + ".github" + ext


def _GenerateExperimentFiles(args, mode):
    if mode == "test":
        _EXPERIMENTS_DEFS = (
            "test/core/experiments/fixtures/test_experiments.yaml"
        )
        _EXPERIMENTS_ROLLOUTS = (
            "test/core/experiments/fixtures/test_experiments_rollout.yaml"
        )
        _EXPERIMENTS_HDR_FILE = "test/core/experiments/fixtures/experiments.h"
        _EXPERIMENTS_SRC_FILE = "test/core/experiments/fixtures/experiments.cc"
        _EXPERIMENTS_BZL_FILE = "bazel/test_experiments.bzl"
    else:
        _EXPERIMENTS_DEFS = "src/core/lib/experiments/experiments.yaml"
        _EXPERIMENTS_ROLLOUTS = "src/core/lib/experiments/rollouts.yaml"
        _EXPERIMENTS_HDR_FILE = "src/core/lib/experiments/experiments.h"
        _EXPERIMENTS_SRC_FILE = "src/core/lib/experiments/experiments.cc"
        _EXPERIMENTS_BZL_FILE = "bazel/experiments.bzl"
        if "/google3/" in REPO_ROOT:
            _EXPERIMENTS_ROLLOUTS = _InjectGithubPath(_EXPERIMENTS_ROLLOUTS)
            _EXPERIMENTS_HDR_FILE = _InjectGithubPath(_EXPERIMENTS_HDR_FILE)
            _EXPERIMENTS_SRC_FILE = _InjectGithubPath(_EXPERIMENTS_SRC_FILE)
            _EXPERIMENTS_BZL_FILE = _InjectGithubPath(_EXPERIMENTS_BZL_FILE)

    with open(_EXPERIMENTS_DEFS) as f:
        attrs = yaml.safe_load(f.read())

    if not exp.AreExperimentsOrdered(attrs):
        print("Experiments are not ordered")
        sys.exit(1)

    with open(_EXPERIMENTS_ROLLOUTS) as f:
        rollouts = yaml.safe_load(f.read())

    if not exp.AreExperimentsOrdered(rollouts):
        print("Rollouts are not ordered")
        sys.exit(1)

    compiler = exp.ExperimentsCompiler(
        DEFAULTS,
        FINAL_RETURN,
        FINAL_DEFINE,
        PLATFORMS_DEFINE,
        BZL_LIST_FOR_DEFAULTS,
    )

    experiment_annotation = "gRPC Experiments: "
    for attr in attrs:
        exp_definition = exp.ExperimentDefinition(attr)
        if not exp_definition.IsValid(args.check):
            sys.exit(1)
        experiment_annotation += exp_definition.name + ":0,"
        if not compiler.AddExperimentDefinition(exp_definition):
            print("Experiment = %s ERROR adding" % exp_definition.name)
            sys.exit(1)

    if len(experiment_annotation) > 2000:
        print("comma-delimited string of experiments is too long")
        sys.exit(1)

    for rollout_attr in rollouts:
        if not compiler.AddRolloutSpecification(rollout_attr):
            print("ERROR adding rollout spec")
            sys.exit(1)

    if mode != "test" and args.no_dbg_experiments:
        print("Ensuring no debug experiments are configured")
        compiler.EnsureNoDebugExperiments()

    print(f"Mode = {mode} Generating experiments headers")
    compiler.GenerateExperimentsHdr(_EXPERIMENTS_HDR_FILE, mode)

    print(f"Mode = {mode} Generating experiments srcs")
    compiler.GenerateExperimentsSrc(
        _EXPERIMENTS_SRC_FILE, _EXPERIMENTS_HDR_FILE, mode
    )

    print("Generating experiments.bzl")
    compiler.GenExperimentsBzl(mode, _EXPERIMENTS_BZL_FILE)
    if mode == "test":
        print("Generating experiments tests")
        compiler.GenTest(
            os.path.join(REPO_ROOT, "test/core/experiments/experiments_test.cc")
        )


_GenerateExperimentFiles(args, "production")
_GenerateExperimentFiles(args, "test")



#!/usr/bin/env python3

# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys

script_dir = os.path.dirname(os.path.abspath(__file__))
sys.path.append(script_dir)

import scenario_generator_helper as gen

gen.generate_scenarios_bzl(
    gen.generate_qps_json_driver_scenarios(),
    os.path.join(script_dir, "qps_json_driver_scenarios.bzl"),
    "QPS_JSON_DRIVER_SCENARIOS",
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/common/aws/v3/credential_provider.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/common/aws/v3/credential_provider.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from udpa.annotations import sensitive_pb2 as udpa_dot_annotations_dot_sensitive__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n8envoy/extensions/common/aws/v3/credential_provider.proto\x12\x1e\x65nvoy.extensions.common.aws.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a\x1egoogle/protobuf/duration.proto\x1a udpa/annotations/sensitive.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xef\x07\n\x15\x41wsCredentialProvider\x12{\n&assume_role_with_web_identity_provider\x18\x01 \x01(\x0b\x32K.envoy.extensions.common.aws.v3.AssumeRoleWithWebIdentityCredentialProvider\x12S\n\x11inline_credential\x18\x02 \x01(\x0b\x32\x38.envoy.extensions.common.aws.v3.InlineCredentialProvider\x12\x64\n\x19\x63redentials_file_provider\x18\x03 \x01(\x0b\x32\x41.envoy.extensions.common.aws.v3.CredentialsFileCredentialProvider\x12(\n custom_credential_provider_chain\x18\x04 \x01(\x08\x12r\n&iam_roles_anywhere_credential_provider\x18\x05 \x01(\x0b\x32\x42.envoy.extensions.common.aws.v3.IAMRolesAnywhereCredentialProvider\x12\\\n\x1a\x63onfig_credential_provider\x18\x06 \x01(\x0b\x32\x38.envoy.extensions.common.aws.v3.ConfigCredentialProvider\x12\x62\n\x1d\x63ontainer_credential_provider\x18\x07 \x01(\x0b\x32;.envoy.extensions.common.aws.v3.ContainerCredentialProvider\x12\x66\n\x1f\x65nvironment_credential_provider\x18\x08 \x01(\x0b\x32=.envoy.extensions.common.aws.v3.EnvironmentCredentialProvider\x12o\n$instance_profile_credential_provider\x18\t \x01(\x0b\x32\x41.envoy.extensions.common.aws.v3.InstanceProfileCredentialProvider\x12\x65\n\x1f\x61ssume_role_credential_provider\x18\n \x01(\x0b\x32<.envoy.extensions.common.aws.v3.AssumeRoleCredentialProvider\"\x83\x01\n\x18InlineCredentialProvider\x12\x1e\n\raccess_key_id\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12(\n\x11secret_access_key\x18\x02 \x01(\tB\r\xfa\x42\x04r\x02\x10\x01\xb8\xb7\x8b\xa4\x02\x01\x12\x1d\n\rsession_token\x18\x03 \x01(\tB\x06\xb8\xb7\x8b\xa4\x02\x01\"\xb5\x01\n+AssumeRoleWithWebIdentityCredentialProvider\x12P\n\x1eweb_identity_token_data_source\x18\x01 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\x06\xb8\xb7\x8b\xa4\x02\x01\x12\x19\n\x08role_arn\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x19\n\x11role_session_name\x18\x03 \x01(\t\"\x7f\n!CredentialsFileCredentialProvider\x12I\n\x17\x63redentials_data_source\x18\x01 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\x06\xb8\xb7\x8b\xa4\x02\x01\x12\x0f\n\x07profile\x18\x02 \x01(\t\"\xa8\x03\n\"IAMRolesAnywhereCredentialProvider\x12\x19\n\x08role_arn\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12?\n\x0b\x63\x65rtificate\x18\x02 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12;\n\x11\x63\x65rtificate_chain\x18\x03 \x01(\x0b\x32 .envoy.config.core.v3.DataSource\x12\x45\n\x0bprivate_key\x18\x04 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\x0e\xfa\x42\x05\x8a\x01\x02\x10\x01\xb8\xb7\x8b\xa4\x02\x01\x12!\n\x10trust_anchor_arn\x18\x05 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x1c\n\x0bprofile_arn\x18\x06 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x19\n\x11role_session_name\x18\x07 \x01(\t\x12\x46\n\x10session_duration\x18\x08 \x01(\x0b\x32\x19.google.protobuf.DurationB\x11\xfa\x42\x0e\xaa\x01\x0b\"\x04\x08\xc0\xd1\x02\x32\x03\x08\x84\x07\"\x1a\n\x18\x43onfigCredentialProvider\"\x1d\n\x1b\x43ontainerCredentialProvider\"\x1f\n\x1d\x45nvironmentCredentialProvider\"#\n!InstanceProfileCredentialProvider\"\x85\x02\n\x1c\x41ssumeRoleCredentialProvider\x12\x19\n\x08role_arn\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x19\n\x11role_session_name\x18\x02 \x01(\t\x12\x13\n\x0b\x65xternal_id\x18\x03 \x01(\t\x12\x46\n\x10session_duration\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationB\x11\xfa\x42\x0e\xaa\x01\x0b\"\x04\x08\xc0\xd1\x02\x32\x03\x08\x84\x07\x12R\n\x13\x63redential_provider\x18\x05 \x01(\x0b\x32\x35.envoy.extensions.common.aws.v3.AwsCredentialProviderB\x9e\x01\n,io.envoyproxy.envoy.extensions.common.aws.v3B\x17\x43redentialProviderProtoP\x01ZKgithub.com/envoyproxy/go-control-plane/envoy/extensions/common/aws/v3;awsv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.common.aws.v3.credential_provider_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n,io.envoyproxy.envoy.extensions.common.aws.v3B\027CredentialProviderProtoP\001ZKgithub.com/envoyproxy/go-control-plane/envoy/extensions/common/aws/v3;awsv3\272\200\310\321\006\002\020\002'
  _globals['_INLINECREDENTIALPROVIDER'].fields_by_name['access_key_id']._loaded_options = None
  _globals['_INLINECREDENTIALPROVIDER'].fields_by_name['access_key_id']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_INLINECREDENTIALPROVIDER'].fields_by_name['secret_access_key']._loaded_options = None
  _globals['_INLINECREDENTIALPROVIDER'].fields_by_name['secret_access_key']._serialized_options = b'\372B\004r\002\020\001\270\267\213\244\002\001'
  _globals['_INLINECREDENTIALPROVIDER'].fields_by_name['session_token']._loaded_options = None
  _globals['_INLINECREDENTIALPROVIDER'].fields_by_name['session_token']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_ASSUMEROLEWITHWEBIDENTITYCREDENTIALPROVIDER'].fields_by_name['web_identity_token_data_source']._loaded_options = None
  _globals['_ASSUMEROLEWITHWEBIDENTITYCREDENTIALPROVIDER'].fields_by_name['web_identity_token_data_source']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_ASSUMEROLEWITHWEBIDENTITYCREDENTIALPROVIDER'].fields_by_name['role_arn']._loaded_options = None
  _globals['_ASSUMEROLEWITHWEBIDENTITYCREDENTIALPROVIDER'].fields_by_name['role_arn']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CREDENTIALSFILECREDENTIALPROVIDER'].fields_by_name['credentials_data_source']._loaded_options = None
  _globals['_CREDENTIALSFILECREDENTIALPROVIDER'].fields_by_name['credentials_data_source']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_IAMROLESANYWHERECREDENTIALPROVIDER'].fields_by_name['role_arn']._loaded_options = None
  _globals['_IAMROLESANYWHERECREDENTIALPROVIDER'].fields_by_name['role_arn']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_IAMROLESANYWHERECREDENTIALPROVIDER'].fields_by_name['certificate']._loaded_options = None
  _globals['_IAMROLESANYWHERECREDENTIALPROVIDER'].fields_by_name['certificate']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_IAMROLESANYWHERECREDENTIALPROVIDER'].fields_by_name['private_key']._loaded_options = None
  _globals['_IAMROLESANYWHERECREDENTIALPROVIDER'].fields_by_name['private_key']._serialized_options = b'\372B\005\212\001\002\020\001\270\267\213\244\002\001'
  _globals['_IAMROLESANYWHERECREDENTIALPROVIDER'].fields_by_name['trust_anchor_arn']._loaded_options = None
  _globals['_IAMROLESANYWHERECREDENTIALPROVIDER'].fields_by_name['trust_anchor_arn']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_IAMROLESANYWHERECREDENTIALPROVIDER'].fields_by_name['profile_arn']._loaded_options = None
  _globals['_IAMROLESANYWHERECREDENTIALPROVIDER'].fields_by_name['profile_arn']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_IAMROLESANYWHERECREDENTIALPROVIDER'].fields_by_name['session_duration']._loaded_options = None
  _globals['_IAMROLESANYWHERECREDENTIALPROVIDER'].fields_by_name['session_duration']._serialized_options = b'\372B\016\252\001\013\"\004\010\300\321\0022\003\010\204\007'
  _globals['_ASSUMEROLECREDENTIALPROVIDER'].fields_by_name['role_arn']._loaded_options = None
  _globals['_ASSUMEROLECREDENTIALPROVIDER'].fields_by_name['role_arn']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_ASSUMEROLECREDENTIALPROVIDER'].fields_by_name['session_duration']._loaded_options = None
  _globals['_ASSUMEROLECREDENTIALPROVIDER'].fields_by_name['session_duration']._serialized_options = b'\372B\016\252\001\013\"\004\010\300\321\0022\003\010\204\007'
  _globals['_AWSCREDENTIALPROVIDER']._serialized_start=248
  _globals['_AWSCREDENTIALPROVIDER']._serialized_end=1255
  _globals['_INLINECREDENTIALPROVIDER']._serialized_start=1258
  _globals['_INLINECREDENTIALPROVIDER']._serialized_end=1389
  _globals['_ASSUMEROLEWITHWEBIDENTITYCREDENTIALPROVIDER']._serialized_start=1392
  _globals['_ASSUMEROLEWITHWEBIDENTITYCREDENTIALPROVIDER']._serialized_end=1573
  _globals['_CREDENTIALSFILECREDENTIALPROVIDER']._serialized_start=1575
  _globals['_CREDENTIALSFILECREDENTIALPROVIDER']._serialized_end=1702
  _globals['_IAMROLESANYWHERECREDENTIALPROVIDER']._serialized_start=1705
  _globals['_IAMROLESANYWHERECREDENTIALPROVIDER']._serialized_end=2129
  _globals['_CONFIGCREDENTIALPROVIDER']._serialized_start=2131
  _globals['_CONFIGCREDENTIALPROVIDER']._serialized_end=2157
  _globals['_CONTAINERCREDENTIALPROVIDER']._serialized_start=2159
  _globals['_CONTAINERCREDENTIALPROVIDER']._serialized_end=2188
  _globals['_ENVIRONMENTCREDENTIALPROVIDER']._serialized_start=2190
  _globals['_ENVIRONMENTCREDENTIALPROVIDER']._serialized_end=2221
  _globals['_INSTANCEPROFILECREDENTIALPROVIDER']._serialized_start=2223
  _globals['_INSTANCEPROFILECREDENTIALPROVIDER']._serialized_end=2258
  _globals['_ASSUMEROLECREDENTIALPROVIDER']._serialized_start=2261
  _globals['_ASSUMEROLECREDENTIALPROVIDER']._serialized_end=2522
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: google/api/servicecontrol/v2/service_controller.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'google/api/servicecontrol/v2/service_controller.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.api import client_pb2 as google_dot_api_dot_client__pb2
from google.rpc.context import attribute_context_pb2 as google_dot_rpc_dot_context_dot_attribute__context__pb2
from google.rpc import status_pb2 as google_dot_rpc_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5google/api/servicecontrol/v2/service_controller.proto\x12\x1cgoogle.api.servicecontrol.v2\x1a\x1cgoogle/api/annotations.proto\x1a\x17google/api/client.proto\x1a*google/rpc/context/attribute_context.proto\x1a\x17google/rpc/status.proto\"\xc7\x01\n\x0c\x43heckRequest\x12\x14\n\x0cservice_name\x18\x01 \x01(\t\x12\x19\n\x11service_config_id\x18\x02 \x01(\t\x12\x38\n\nattributes\x18\x03 \x01(\x0b\x32$.google.rpc.context.AttributeContext\x12=\n\tresources\x18\x04 \x03(\x0b\x32*.google.api.servicecontrol.v2.ResourceInfo\x12\r\n\x05\x66lags\x18\x05 \x01(\t\"c\n\x0cResourceInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\x12\n\npermission\x18\x03 \x01(\t\x12\x11\n\tcontainer\x18\x04 \x01(\t\x12\x10\n\x08location\x18\x05 \x01(\t\"\xae\x01\n\rCheckResponse\x12\"\n\x06status\x18\x01 \x01(\x0b\x32\x12.google.rpc.Status\x12I\n\x07headers\x18\x02 \x03(\x0b\x32\x38.google.api.servicecontrol.v2.CheckResponse.HeadersEntry\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"z\n\rReportRequest\x12\x14\n\x0cservice_name\x18\x01 \x01(\t\x12\x19\n\x11service_config_id\x18\x02 \x01(\t\x12\x38\n\noperations\x18\x03 \x03(\x0b\x32$.google.rpc.context.AttributeContext\"\x10\n\x0eReportResponse\"Q\n\x10ResourceInfoList\x12=\n\tresources\x18\x01 \x03(\x0b\x32*.google.api.servicecontrol.v2.ResourceInfo2\xbc\x03\n\x11ServiceController\x12\x8e\x01\n\x05\x43heck\x12*.google.api.servicecontrol.v2.CheckRequest\x1a+.google.api.servicecontrol.v2.CheckResponse\",\x82\xd3\xe4\x93\x02&\"!/v2/services/{service_name}:check:\x01*\x12\x92\x01\n\x06Report\x12+.google.api.servicecontrol.v2.ReportRequest\x1a,.google.api.servicecontrol.v2.ReportResponse\"-\x82\xd3\xe4\x93\x02\'\"\"/v2/services/{service_name}:report:\x01*\x1a\x80\x01\xca\x41\x1dservicecontrol.googleapis.com\xd2\x41]https://www.googleapis.com/auth/cloud-platform,https://www.googleapis.com/auth/servicecontrolB\xee\x01\n com.google.api.servicecontrol.v2B\x16ServiceControllerProtoP\x01ZJgoogle.golang.org/genproto/googleapis/api/servicecontrol/v2;servicecontrol\xaa\x02\x1eGoogle.Cloud.ServiceControl.V2\xca\x02\x1eGoogle\\Cloud\\ServiceControl\\V2\xea\x02!Google::Cloud::ServiceControl::V2b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'google.api.servicecontrol.v2.service_controller_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n com.google.api.servicecontrol.v2B\026ServiceControllerProtoP\001ZJgoogle.golang.org/genproto/googleapis/api/servicecontrol/v2;servicecontrol\252\002\036Google.Cloud.ServiceControl.V2\312\002\036Google\\Cloud\\ServiceControl\\V2\352\002!Google::Cloud::ServiceControl::V2'
  _globals['_CHECKRESPONSE_HEADERSENTRY']._loaded_options = None
  _globals['_CHECKRESPONSE_HEADERSENTRY']._serialized_options = b'8\001'
  _globals['_SERVICECONTROLLER']._loaded_options = None
  _globals['_SERVICECONTROLLER']._serialized_options = b'\312A\035servicecontrol.googleapis.com\322A]https://www.googleapis.com/auth/cloud-platform,https://www.googleapis.com/auth/servicecontrol'
  _globals['_SERVICECONTROLLER'].methods_by_name['Check']._loaded_options = None
  _globals['_SERVICECONTROLLER'].methods_by_name['Check']._serialized_options = b'\202\323\344\223\002&\"!/v2/services/{service_name}:check:\001*'
  _globals['_SERVICECONTROLLER'].methods_by_name['Report']._loaded_options = None
  _globals['_SERVICECONTROLLER'].methods_by_name['Report']._serialized_options = b'\202\323\344\223\002\'\"\"/v2/services/{service_name}:report:\001*'
  _globals['_CHECKREQUEST']._serialized_start=212
  _globals['_CHECKREQUEST']._serialized_end=411
  _globals['_RESOURCEINFO']._serialized_start=413
  _globals['_RESOURCEINFO']._serialized_end=512
  _globals['_CHECKRESPONSE']._serialized_start=515
  _globals['_CHECKRESPONSE']._serialized_end=689
  _globals['_CHECKRESPONSE_HEADERSENTRY']._serialized_start=643
  _globals['_CHECKRESPONSE_HEADERSENTRY']._serialized_end=689
  _globals['_REPORTREQUEST']._serialized_start=691
  _globals['_REPORTREQUEST']._serialized_end=813
  _globals['_REPORTRESPONSE']._serialized_start=815
  _globals['_REPORTRESPONSE']._serialized_end=831
  _globals['_RESOURCEINFOLIST']._serialized_start=833
  _globals['_RESOURCEINFOLIST']._serialized_end=914
  _globals['_SERVICECONTROLLER']._serialized_start=917
  _globals['_SERVICECONTROLLER']._serialized_end=1361
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3
# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Uploads RBE results to BigQuery"""

import argparse
import json
import os
import ssl
import sys
import urllib.error
import urllib.parse
import urllib.request
import uuid

sys.path.append(os.path.dirname(os.path.abspath(__file__)))
import utils_common

gcp_utils_dir = os.path.abspath(
    os.path.join(os.path.dirname(__file__), "../../gcp/utils")
)
sys.path.append(gcp_utils_dir)
import big_query_utils

_DATASET_ID = "jenkins_test_results"
_DESCRIPTION = "Test results from master RBE builds on Kokoro"
# 365 days in milliseconds
_EXPIRATION_MS = 365 * 24 * 60 * 60 * 1000
_PARTITION_TYPE = "DAY"
_PROJECT_ID = "grpc-testing"
_RESULTS_SCHEMA = [
    ("job_name", "STRING", "Name of Kokoro job"),
    ("build_id", "INTEGER", "Build ID of Kokoro job"),
    ("build_url", "STRING", "URL of Kokoro build"),
    ("test_target", "STRING", "Bazel target path"),
    ("test_class_name", "STRING", "Name of test class"),
    ("test_case", "STRING", "Name of test case"),
    ("result", "STRING", "Test or build result"),
    ("timestamp", "TIMESTAMP", "Timestamp of test run"),
    ("duration", "FLOAT", "Duration of the test run"),
]
_TABLE_ID = "rbe_test_results"


def _get_api_key():
    """Returns string with API key to access ResultStore.
    Intended to be used in Kokoro environment."""
    api_key_directory = os.getenv("KOKORO_GFILE_DIR")
    api_key_file = os.path.join(api_key_directory, "resultstore_api_key")
    assert os.path.isfile(api_key_file), (
        "Must add --api_key arg if not on "
        "Kokoro or Kokoro environment is not set up properly."
    )
    with open(api_key_file, "r") as f:
        return f.read().replace("\n", "")


def _get_invocation_id():
    """Returns String of Bazel invocation ID. Intended to be used in
    Kokoro environment."""
    bazel_id_directory = os.getenv("KOKORO_ARTIFACTS_DIR")
    bazel_id_file = os.path.join(bazel_id_directory, "bazel_invocation_ids")
    assert os.path.isfile(bazel_id_file), (
        "bazel_invocation_ids file, written "
        "by RBE initialization script, expected but not found."
    )
    with open(bazel_id_file, "r") as f:
        return f.read().replace("\n", "")


def _parse_test_duration(duration_str):
    """Parse test duration string in '123.567s' format"""
    try:
        if duration_str.endswith("s"):
            duration_str = duration_str[:-1]
        return float(duration_str)
    except:
        return None


def _upload_results_to_bq(rows):
    """Upload test results to a BQ table.

    Args:
        rows: A list of dictionaries containing data for each row to insert
    """
    bq = big_query_utils.create_big_query()
    big_query_utils.create_partitioned_table(
        bq,
        _PROJECT_ID,
        _DATASET_ID,
        _TABLE_ID,
        _RESULTS_SCHEMA,
        _DESCRIPTION,
        partition_type=_PARTITION_TYPE,
        expiration_ms=_EXPIRATION_MS,
    )

    max_retries = 3
    for attempt in range(max_retries):
        if big_query_utils.insert_rows(
            bq, _PROJECT_ID, _DATASET_ID, _TABLE_ID, rows
        ):
            break
        else:
            if attempt < max_retries - 1:
                print("Error uploading result to bigquery, will retry.")
            else:
                print(
                    "Error uploading result to bigquery, all attempts failed."
                )
                sys.exit(1)


def _get_resultstore_data(api_key, invocation_id):
    """Returns dictionary of test results by querying ResultStore API.
    Args:
        api_key: String of ResultStore API key
        invocation_id: String of ResultStore invocation ID to results from
    """
    all_actions = []
    page_token = ""
    # ResultStore's API returns data on a limited number of tests. When we exceed
    # that limit, the 'nextPageToken' field is included in the request to get
    # subsequent data, so keep requesting until 'nextPageToken' field is omitted.
    while True:
        req = urllib.request.Request(
            url="https://resultstore.googleapis.com/v2/invocations/%s/targets/-/configuredTargets/-/actions?key=%s&pageToken=%s&fields=next_page_token,actions.id,actions.status_attributes,actions.timing,actions.test_action"
            % (invocation_id, api_key, page_token),
            headers={"Content-Type": "application/json"},
        )
        ctx_dict = {}
        if os.getenv("PYTHONHTTPSVERIFY") == "0":
            ctx = ssl.create_default_context()
            ctx.check_hostname = False
            ctx.verify_mode = ssl.CERT_NONE
            ctx_dict = {"context": ctx}
        raw_resp = urllib.request.urlopen(req, **ctx_dict).read()
        decoded_resp = (
            raw_resp
            if isinstance(raw_resp, str)
            else raw_resp.decode("utf-8", "ignore")
        )
        results = json.loads(decoded_resp)
        # ResultStore API may omit the 'actions' field if no actions are found
        # (e.g., in highly cached small runs) or due to eventual consistency
        # delays.
        all_actions.extend(results.get("actions", []))
        if "nextPageToken" not in results:
            break
        page_token = results["nextPageToken"]
    return all_actions


if __name__ == "__main__":
    # Arguments are necessary if running in a non-Kokoro environment.
    argp = argparse.ArgumentParser(
        description=(
            "Fetches results for given RBE invocation and uploads them to"
            " BigQuery table."
        )
    )
    argp.add_argument(
        "--api_key",
        default="",
        type=str,
        help="The API key to read from ResultStore API",
    )
    argp.add_argument(
        "--invocation_id",
        default="",
        type=str,
        help="UUID of bazel invocation to fetch.",
    )
    argp.add_argument(
        "--bq_dump_file",
        default=None,
        type=str,
        help="Dump JSON data to file just before uploading",
    )
    argp.add_argument(
        "--resultstore_dump_file",
        default=None,
        type=str,
        help="Dump JSON data as received from ResultStore API",
    )
    argp.add_argument(
        "--skip_upload",
        default=False,
        action="store_const",
        const=True,
        help="Skip uploading to bigquery",
    )
    args = argp.parse_args()

    api_key = args.api_key or _get_api_key()
    invocation_id = args.invocation_id or _get_invocation_id()
    resultstore_actions = _get_resultstore_data(api_key, invocation_id)

    if args.resultstore_dump_file:
        with open(args.resultstore_dump_file, "w") as f:
            json.dump(resultstore_actions, f, indent=4, sort_keys=True)
        print(
            ("Dumped resultstore data to file %s" % args.resultstore_dump_file)
        )

    # google.devtools.resultstore.v2.Action schema:
    # https://github.com/googleapis/googleapis/blob/master/google/devtools/resultstore/v2/action.proto
    bq_rows = []
    for index, action in enumerate(resultstore_actions):
        # Filter out non-test related data, such as build results.
        if "testAction" not in action:
            continue
        # Some test results contain the fileProcessingErrors field, which indicates
        # an issue with parsing results individual test cases.
        if "fileProcessingErrors" in action:
            test_cases = [
                {
                    "testCase": {
                        "caseName": str(action["id"]["actionId"]),
                    }
                }
            ]
        # Test timeouts have a different dictionary structure compared to pass and
        # fail results.
        elif action["statusAttributes"]["status"] == "TIMED_OUT":
            test_cases = [
                {
                    "testCase": {
                        "caseName": str(action["id"]["actionId"]),
                        "timedOut": True,
                    }
                }
            ]
        # When RBE believes its infrastructure is failing, it will abort and
        # mark running tests as UNKNOWN. These infrastructure failures may be
        # related to our tests, so we should investigate if specific tests are
        # repeatedly being marked as UNKNOWN.
        elif action["statusAttributes"]["status"] == "UNKNOWN":
            test_cases = [
                {
                    "testCase": {
                        "caseName": str(action["id"]["actionId"]),
                        "unknown": True,
                    }
                }
            ]
            # Take the timestamp from the previous action, which should be
            # a close approximation.
            action["timing"] = {
                "startTime": resultstore_actions[index - 1]["timing"][
                    "startTime"
                ]
            }
        elif "testSuite" not in action["testAction"]:
            continue
        elif "tests" not in action["testAction"]["testSuite"]:
            continue
        else:
            test_cases = []
            for tests_item in action["testAction"]["testSuite"]["tests"]:
                test_cases += tests_item["testSuite"]["tests"]

        for test_case in test_cases:
            if any(s in test_case["testCase"] for s in ["errors", "failures"]):
                result = "FAILED"
            elif "timedOut" in test_case["testCase"]:
                result = "TIMEOUT"
            elif "unknown" in test_case["testCase"]:
                result = "UNKNOWN"
            else:
                result = "PASSED"
            try:
                bq_rows.append(
                    {
                        "insertId": str(uuid.uuid4()),
                        "json": {
                            "job_name": os.getenv("KOKORO_JOB_NAME"),
                            "build_id": os.getenv("KOKORO_BUILD_NUMBER"),
                            "build_url": "https://source.cloud.google.com/results/invocations/%s"
                            % invocation_id,
                            "test_target": action["id"]["targetId"],
                            "test_class_name": test_case["testCase"].get(
                                "className", ""
                            ),
                            "test_case": test_case["testCase"]["caseName"],
                            "result": result,
                            "timestamp": action["timing"]["startTime"],
                        },
                    }
                    | (
                        {
                            "duration": _parse_test_duration(
                                action["timing"]["duration"]
                            )
                        }
                        if "duration" in action["timing"]
                        else {}
                    )
                )
            except Exception as e:
                print(("Failed to parse test result. Error: %s" % str(e)))
                print((json.dumps(action, indent=4)))
                bq_rows.append(
                    {
                        "insertId": str(uuid.uuid4()),
                        "json": {
                            "job_name": os.getenv("KOKORO_JOB_NAME"),
                            "build_id": os.getenv("KOKORO_BUILD_NUMBER"),
                            "build_url": "https://source.cloud.google.com/results/invocations/%s"
                            % invocation_id,
                            "test_target": action["id"]["targetId"],
                            "test_class_name": "N/A",
                            "test_case": "N/A",
                            "result": "UNPARSABLE",
                            "timestamp": action["timing"]["startTime"],
                        },
                    }
                )

    if args.bq_dump_file:
        with open(args.bq_dump_file, "w") as f:
            json.dump(bq_rows, f, indent=4, sort_keys=True)
        print(("Dumped BQ data to file %s" % args.bq_dump_file))

    if not args.skip_upload and utils_common.should_upload_results_on_ci():
        # BigQuery sometimes fails with large uploads, so batch 1,000 rows at a time.
        MAX_ROWS = 1000
        for i in range(0, len(bq_rows), MAX_ROWS):
            _upload_results_to_bq(bq_rows[i : i + MAX_ROWS])
    else:
        print("Skipped upload to bigquery.")



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Porting auth context tests from sync stack."""

import logging
import pickle
import unittest

import grpc
from grpc.experimental import aio
from grpc.experimental import session_cache

from tests.unit import resources
from tests_aio.unit._test_base import AioTestBase

_REQUEST = b"\x00\x00\x00"
_RESPONSE = b"\x00\x00\x00"

_UNARY_UNARY = "/test/UnaryUnary"

_SERVER_HOST_OVERRIDE = "foo.test.google.fr"
_CLIENT_IDS = (
    b"*.test.google.fr",
    b"waterzooi.test.google.be",
    b"*.test.youtube.com",
    b"192.168.1.3",
)
_ID = "id"
_ID_KEY = "id_key"
_AUTH_CTX = "auth_ctx"

_PRIVATE_KEY = resources.private_key()
_CERTIFICATE_CHAIN = resources.certificate_chain()
_TEST_ROOT_CERTIFICATES = resources.test_root_certificates()
_SERVER_CERTS = ((_PRIVATE_KEY, _CERTIFICATE_CHAIN),)
_PROPERTY_OPTIONS = (
    (
        "grpc.ssl_target_name_override",
        _SERVER_HOST_OVERRIDE,
    ),
)


async def handle_unary_unary(
    unused_request: bytes, servicer_context: aio.ServicerContext
):
    return pickle.dumps(
        {
            _ID: servicer_context.peer_identities(),
            _ID_KEY: servicer_context.peer_identity_key(),
            _AUTH_CTX: servicer_context.auth_context(),
        }
    )


class TestAuthContext(AioTestBase):
    async def test_insecure(self):
        handler = grpc.method_handlers_generic_handler(
            "test",
            {
                "UnaryUnary": grpc.unary_unary_rpc_method_handler(
                    handle_unary_unary
                )
            },
        )
        server = aio.server()
        server.add_generic_rpc_handlers((handler,))
        port = server.add_insecure_port("[::]:0")
        await server.start()

        async with aio.insecure_channel("localhost:%d" % port) as channel:
            response = await channel.unary_unary(_UNARY_UNARY)(_REQUEST)
        await server.stop(None)

        auth_data = pickle.loads(response)
        self.assertIsNone(auth_data[_ID])
        self.assertIsNone(auth_data[_ID_KEY])
        self.assertDictEqual(
            {
                "security_level": [b"TSI_SECURITY_NONE"],
                "transport_security_type": [b"insecure"],
            },
            auth_data[_AUTH_CTX],
        )

    async def test_secure_no_cert(self):
        handler = grpc.method_handlers_generic_handler(
            "test",
            {
                "UnaryUnary": grpc.unary_unary_rpc_method_handler(
                    handle_unary_unary
                )
            },
        )
        server = aio.server()
        server.add_generic_rpc_handlers((handler,))
        server_cred = grpc.ssl_server_credentials(_SERVER_CERTS)
        port = server.add_secure_port("[::]:0", server_cred)
        await server.start()

        channel_creds = grpc.ssl_channel_credentials(
            root_certificates=_TEST_ROOT_CERTIFICATES
        )
        channel = aio.secure_channel(
            "localhost:{}".format(port),
            channel_creds,
            options=_PROPERTY_OPTIONS,
        )
        response = await channel.unary_unary(_UNARY_UNARY)(_REQUEST)
        await channel.close()
        await server.stop(None)

        auth_data = pickle.loads(response)
        self.assertIsNone(auth_data[_ID])
        self.assertIsNone(auth_data[_ID_KEY])
        self.assertDictEqual(
            {
                "security_level": [b"TSI_PRIVACY_AND_INTEGRITY"],
                "transport_security_type": [b"ssl"],
                "ssl_session_reused": [b"false"],
            },
            auth_data[_AUTH_CTX],
        )

    async def test_secure_client_cert(self):
        handler = grpc.method_handlers_generic_handler(
            "test",
            {
                "UnaryUnary": grpc.unary_unary_rpc_method_handler(
                    handle_unary_unary
                )
            },
        )
        server = aio.server()
        server.add_generic_rpc_handlers((handler,))
        server_cred = grpc.ssl_server_credentials(
            _SERVER_CERTS,
            root_certificates=_TEST_ROOT_CERTIFICATES,
            require_client_auth=True,
        )
        port = server.add_secure_port("[::]:0", server_cred)
        await server.start()

        channel_creds = grpc.ssl_channel_credentials(
            root_certificates=_TEST_ROOT_CERTIFICATES,
            private_key=_PRIVATE_KEY,
            certificate_chain=_CERTIFICATE_CHAIN,
        )
        channel = aio.secure_channel(
            "localhost:{}".format(port),
            channel_creds,
            options=_PROPERTY_OPTIONS,
        )

        response = await channel.unary_unary(_UNARY_UNARY)(_REQUEST)
        await channel.close()
        await server.stop(None)

        auth_data = pickle.loads(response)
        auth_ctx = auth_data[_AUTH_CTX]
        self.assertCountEqual(_CLIENT_IDS, auth_data[_ID])
        self.assertEqual("x509_subject_alternative_name", auth_data[_ID_KEY])
        self.assertSequenceEqual([b"ssl"], auth_ctx["transport_security_type"])
        self.assertSequenceEqual(
            [b"*.test.google.com"], auth_ctx["x509_common_name"]
        )

    async def _do_one_shot_client_rpc(
        self, channel_creds, channel_options, port, expect_ssl_session_reused
    ):
        channel = aio.secure_channel(
            "localhost:{}".format(port), channel_creds, options=channel_options
        )
        response = await channel.unary_unary(_UNARY_UNARY)(_REQUEST)
        auth_data = pickle.loads(response)
        self.assertEqual(
            expect_ssl_session_reused,
            auth_data[_AUTH_CTX]["ssl_session_reused"],
        )
        await channel.close()

    async def test_session_resumption(self):
        # Set up a secure server
        handler = grpc.method_handlers_generic_handler(
            "test",
            {
                "UnaryUnary": grpc.unary_unary_rpc_method_handler(
                    handle_unary_unary
                )
            },
        )
        server = aio.server()
        server.add_generic_rpc_handlers((handler,))
        server_cred = grpc.ssl_server_credentials(_SERVER_CERTS)
        port = server.add_secure_port("[::]:0", server_cred)
        await server.start()

        # Create a cache for TLS session tickets
        cache = session_cache.ssl_session_cache_lru(1)
        channel_creds = grpc.ssl_channel_credentials(
            root_certificates=_TEST_ROOT_CERTIFICATES
        )
        channel_options = _PROPERTY_OPTIONS + (
            ("grpc.ssl_session_cache", cache),
        )

        # Initial connection has no session to resume
        await self._do_one_shot_client_rpc(
            channel_creds,
            channel_options,
            port,
            expect_ssl_session_reused=[b"false"],
        )

        # Subsequent connections resume sessions
        await self._do_one_shot_client_rpc(
            channel_creds,
            channel_options,
            port,
            expect_ssl_session_reused=[b"true"],
        )
        await server.stop(None)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/env python3

# Copyright 2021 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import json
import os
import sys

import yaml

run_tests_root = os.path.abspath(
    os.path.join(os.path.dirname(sys.argv[0]), "../../../tools/run_tests")
)
sys.path.append(run_tests_root)

import performance.scenario_config as scenario_config

_COPYRIGHT = """# Copyright 2021 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""


def _mutate_scenario(scenario_json):
    """Modifies vanilla benchmark scenario config to make it more suitable for running as a unit test."""
    # tweak parameters to get fast test times
    scenario_json = dict(scenario_json)
    scenario_json["warmup_seconds"] = 0
    scenario_json["benchmark_seconds"] = 1
    outstanding_rpcs_divisor = 1
    if (
        scenario_json["client_config"]["client_type"] == "SYNC_CLIENT"
        or scenario_json["server_config"]["server_type"] == "SYNC_SERVER"
    ):
        # reduce the number of threads needed for scenarios that use synchronous API
        outstanding_rpcs_divisor = 10
    scenario_json["client_config"]["outstanding_rpcs_per_channel"] = max(
        1,
        scenario_json["client_config"]["outstanding_rpcs_per_channel"]
        // outstanding_rpcs_divisor,
    )
    # Some scenarios use high channel count since when actually
    # benchmarking, we want to saturate the machine that runs the benchmark.
    # For unit test, this is an overkill.
    max_client_channels = 16
    if scenario_json["client_config"]["rpc_type"] == "STREAMING_FROM_SERVER":
        # streaming from server scenarios tend to have trouble shutting down
        # quickly if there are too many channels.
        max_client_channels = 4

    scenario_json["client_config"]["client_channels"] = min(
        max_client_channels, scenario_json["client_config"]["client_channels"]
    )

    return scenario_config.remove_nonproto_fields(scenario_json)


def generate_json_run_localhost_scenarios():
    return [
        _mutate_scenario(scenario_json)
        for scenario_json in scenario_config.CXXLanguage().scenarios()
        if "scalable" in scenario_json.get("CATEGORIES", [])
    ]


def generate_qps_json_driver_scenarios():
    return [
        _mutate_scenario(scenario_json)
        for scenario_json in scenario_config.CXXLanguage().scenarios()
        if "inproc" in scenario_json.get("CATEGORIES", [])
    ]


def generate_scenarios_bzl(json_scenarios, bzl_filename, bzl_variablename):
    """Generate .bzl file that defines a variable with JSON scenario configs."""
    all_scenarios = []
    for scenario in json_scenarios:
        scenario_name = scenario["name"]
        # argument will be passed as "--scenarios_json" to the test binary
        # the string needs to be quoted in \' to ensure it gets passed as a single argument in shell
        scenarios_json_arg_str = "\\'%s\\'" % json.dumps(
            {"scenarios": [scenario]}
        )
        all_scenarios.append((scenario_name, scenarios_json_arg_str))

    with open(bzl_filename, "w") as f:
        f.write(_COPYRIGHT)
        f.write(
            '"""AUTOGENERATED: configuration of benchmark scenarios to be run'
            ' as bazel test"""\n\n'
        )
        f.write("%s = {\n" % bzl_variablename)
        for scenario in all_scenarios:
            f.write("    \"%s\": '%s',\n" % (scenario[0], scenario[1]))
        f.write("}\n")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/network/generic_proxy/action/v3/action.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/network/generic_proxy/action/v3/action.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.config.route.v3 import route_components_pb2 as envoy_dot_config_dot_route_dot_v3_dot_route__components__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nEenvoy/extensions/filters/network/generic_proxy/action/v3/action.proto\x12\x38\x65nvoy.extensions.filters.network.generic_proxy.action.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a,envoy/config/route/v3/route_components.proto\x1a\x19google/protobuf/any.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xe9\x03\n\x0bRouteAction\x12\x0c\n\x04name\x18\x05 \x01(\t\x12\x11\n\x07\x63luster\x18\x01 \x01(\tH\x00\x12\x43\n\x11weighted_clusters\x18\x02 \x01(\x0b\x32&.envoy.config.route.v3.WeightedClusterH\x00\x12\x30\n\x08metadata\x18\x03 \x01(\x0b\x32\x1e.envoy.config.core.v3.Metadata\x12u\n\x11per_filter_config\x18\x04 \x03(\x0b\x32Z.envoy.extensions.filters.network.generic_proxy.action.v3.RouteAction.PerFilterConfigEntry\x12*\n\x07timeout\x18\x06 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x37\n\x0cretry_policy\x18\x07 \x01(\x0b\x32!.envoy.config.core.v3.RetryPolicy\x1aL\n\x14PerFilterConfigEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any:\x02\x38\x01\x42\x18\n\x11\x63luster_specifier\x12\x03\xf8\x42\x01\x42\xd1\x01\nFio.envoyproxy.envoy.extensions.filters.network.generic_proxy.action.v3B\x0b\x41\x63tionProtoP\x01Zhgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/generic_proxy/action/v3;actionv3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.network.generic_proxy.action.v3.action_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nFio.envoyproxy.envoy.extensions.filters.network.generic_proxy.action.v3B\013ActionProtoP\001Zhgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/generic_proxy/action/v3;actionv3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_ROUTEACTION_PERFILTERCONFIGENTRY']._loaded_options = None
  _globals['_ROUTEACTION_PERFILTERCONFIGENTRY']._serialized_options = b'8\001'
  _globals['_ROUTEACTION'].oneofs_by_name['cluster_specifier']._loaded_options = None
  _globals['_ROUTEACTION'].oneofs_by_name['cluster_specifier']._serialized_options = b'\370B\001'
  _globals['_ROUTEACTION']._serialized_start=359
  _globals['_ROUTEACTION']._serialized_end=848
  _globals['_ROUTEACTION_PERFILTERCONFIGENTRY']._serialized_start=746
  _globals['_ROUTEACTION_PERFILTERCONFIGENTRY']._serialized_end=822
# @@protoc_insertion_point(module_scope)



# Copyright 2022 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import subprocess
import sys
import tempfile


def run_buildozer(buildozer_commands):
    if not buildozer_commands:
        return
    ok_statuses = (0, 3)
    temp = tempfile.NamedTemporaryFile()
    open(temp.name, "w").write("\n".join(buildozer_commands))
    c = ["tools/distrib/buildozer.sh", "-f", temp.name]
    r = subprocess.call(c)
    if r not in ok_statuses:
        print("{} failed with status {}".format(c, r))
        sys.exit(1)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/type/v3/token_bucket.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/type/v3/token_bucket.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n envoy/type/v3/token_bucket.proto\x12\renvoy.type.v3\x1a\x1egoogle/protobuf/duration.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xc7\x01\n\x0bTokenBucket\x12\x1b\n\nmax_tokens\x18\x01 \x01(\rB\x07\xfa\x42\x04*\x02 \x00\x12>\n\x0ftokens_per_fill\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x07\xfa\x42\x04*\x02 \x00\x12<\n\rfill_interval\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationB\n\xfa\x42\x07\xaa\x01\x04\x08\x01*\x00:\x1d\x9a\xc5\x88\x1e\x18\n\x16\x65nvoy.type.TokenBucketBv\n\x1bio.envoyproxy.envoy.type.v3B\x10TokenBucketProtoP\x01Z;github.com/envoyproxy/go-control-plane/envoy/type/v3;typev3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.type.v3.token_bucket_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\033io.envoyproxy.envoy.type.v3B\020TokenBucketProtoP\001Z;github.com/envoyproxy/go-control-plane/envoy/type/v3;typev3\272\200\310\321\006\002\020\002'
  _globals['_TOKENBUCKET'].fields_by_name['max_tokens']._loaded_options = None
  _globals['_TOKENBUCKET'].fields_by_name['max_tokens']._serialized_options = b'\372B\004*\002 \000'
  _globals['_TOKENBUCKET'].fields_by_name['tokens_per_fill']._loaded_options = None
  _globals['_TOKENBUCKET'].fields_by_name['tokens_per_fill']._serialized_options = b'\372B\004*\002 \000'
  _globals['_TOKENBUCKET'].fields_by_name['fill_interval']._loaded_options = None
  _globals['_TOKENBUCKET'].fields_by_name['fill_interval']._serialized_options = b'\372B\007\252\001\004\010\001*\000'
  _globals['_TOKENBUCKET']._loaded_options = None
  _globals['_TOKENBUCKET']._serialized_options = b'\232\305\210\036\030\n\026envoy.type.TokenBucket'
  _globals['_TOKENBUCKET']._serialized_start=207
  _globals['_TOKENBUCKET']._serialized_end=406
# @@protoc_insertion_point(module_scope)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# distutils: language=c++

cimport cpython

import logging
import os
import sys
import threading
import time

import grpc

try:
    import asyncio
except ImportError:
    # TODO(https://github.com/grpc/grpc/issues/19728) Improve how Aio Cython is
    # distributed without breaking none compatible Python versions. For now, if
    # Asyncio package is not available we just skip it.
    pass

# The only copy of Python logger for the Cython extension
_LOGGER = logging.getLogger(__name__)

# TODO(atash): figure out why the coverage tool gets confused about the Cython
# coverage plugin when the following files don't have a '.pxi' suffix.
include "_cygrpc/grpc_string.pyx.pxi"
include "_cygrpc/absl.pyx.pxi"
include "_cygrpc/arguments.pyx.pxi"
include "_cygrpc/call.pyx.pxi"
include "_cygrpc/channel.pyx.pxi"
include "_cygrpc/channelz.pyx.pxi"
include "_cygrpc/csds.pyx.pxi"
include "_cygrpc/credentials.pyx.pxi"
include "_cygrpc/private_key_offload.pyx.pxi"
include "_cygrpc/completion_queue.pyx.pxi"
include "_cygrpc/event.pyx.pxi"
include "_cygrpc/metadata.pyx.pxi"
include "_cygrpc/operation.pyx.pxi"
include "_cygrpc/propagation_bits.pyx.pxi"
include "_cygrpc/records.pyx.pxi"
include "_cygrpc/security.pyx.pxi"
include "_cygrpc/server.pyx.pxi"
include "_cygrpc/tag.pyx.pxi"
include "_cygrpc/time.pyx.pxi"
include "_cygrpc/vtable.pyx.pxi"
include "_cygrpc/_hooks.pyx.pxi"
include "_cygrpc/observability.pyx.pxi"

include "_cygrpc/grpc_gevent.pyx.pxi"

include "_cygrpc/thread.pyx.pxi"

include "_cygrpc/fork.pyx.pxi"

# Following pxi files are part of the Aio module
include "_cygrpc/aio/common.pyx.pxi"
include "_cygrpc/aio/rpc_status.pyx.pxi"
include "_cygrpc/aio/completion_queue.pyx.pxi"
include "_cygrpc/aio/callback_common.pyx.pxi"
include "_cygrpc/aio/grpc_aio.pyx.pxi"
include "_cygrpc/aio/call.pyx.pxi"
include "_cygrpc/aio/channel.pyx.pxi"
include "_cygrpc/aio/server.pyx.pxi"

#
# initialize gRPC
#
cdef _initialize():
  grpc_set_ssl_roots_override_callback(
          <grpc_ssl_roots_override_callback>ssl_roots_override_callback)


_initialize()



# Copyright 2019 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import division

from concurrent import futures
import datetime
import threading
import time
import unittest

import grpc

from tests.unit.framework.common import test_constants

_WAIT_FOR_BLOCKING = datetime.timedelta(seconds=1)


def _block_on_waiting(server, termination_event, timeout=None):
    server.start()
    server.wait_for_termination(timeout=timeout)
    termination_event.set()


class ServerWaitForTerminationTest(unittest.TestCase):
    def test_unblock_by_invoking_stop(self):
        termination_event = threading.Event()
        server = grpc.server(futures.ThreadPoolExecutor())

        wait_thread = threading.Thread(
            target=_block_on_waiting,
            args=(
                server,
                termination_event,
            ),
        )
        wait_thread.daemon = True
        wait_thread.start()
        time.sleep(_WAIT_FOR_BLOCKING.total_seconds())

        server.stop(None)
        termination_event.wait(timeout=test_constants.SHORT_TIMEOUT)
        self.assertTrue(termination_event.is_set())

    def test_unblock_by_del(self):
        termination_event = threading.Event()
        server = grpc.server(futures.ThreadPoolExecutor())

        wait_thread = threading.Thread(
            target=_block_on_waiting,
            args=(
                server,
                termination_event,
            ),
        )
        wait_thread.daemon = True
        wait_thread.start()
        time.sleep(_WAIT_FOR_BLOCKING.total_seconds())

        # Invoke manually here, in Python 2 it will be invoked by GC sometime.
        server.__del__()
        termination_event.wait(timeout=test_constants.SHORT_TIMEOUT)
        self.assertTrue(termination_event.is_set())

    def test_unblock_by_timeout(self):
        termination_event = threading.Event()
        server = grpc.server(futures.ThreadPoolExecutor())

        wait_thread = threading.Thread(
            target=_block_on_waiting,
            args=(
                server,
                termination_event,
                test_constants.SHORT_TIMEOUT / 2,
            ),
        )
        wait_thread.daemon = True
        wait_thread.start()

        termination_event.wait(timeout=test_constants.SHORT_TIMEOUT)
        self.assertTrue(termination_event.is_set())


if __name__ == "__main__":
    unittest.main(verbosity=2)



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test helpers for RPC invocation tests."""

import datetime
import threading

import grpc
from grpc.framework.foundation import logging_pool

from tests.unit import test_common
from tests.unit import thread_pool
from tests.unit.framework.common import test_constants
from tests.unit.framework.common import test_control

_SERIALIZE_REQUEST = lambda bytestring: bytestring * 2
_DESERIALIZE_REQUEST = lambda bytestring: bytestring[len(bytestring) // 2 :]
_SERIALIZE_RESPONSE = lambda bytestring: bytestring * 3
_DESERIALIZE_RESPONSE = lambda bytestring: bytestring[: len(bytestring) // 3]

_SERVICE_NAME = "test"
_UNARY_UNARY = "UnaryUnary"
_UNARY_STREAM = "UnaryStream"
_UNARY_STREAM_NON_BLOCKING = "UnaryStreamNonBlocking"
_STREAM_UNARY = "StreamUnary"
_STREAM_STREAM = "StreamStream"
_STREAM_STREAM_NON_BLOCKING = "StreamStreamNonBlocking"

TIMEOUT_SHORT = datetime.timedelta(seconds=4).total_seconds()


class Callback:
    def __init__(self):
        self._condition = threading.Condition()
        self._value = None
        self._called = False

    def __call__(self, value):
        with self._condition:
            self._value = value
            self._called = True
            self._condition.notify_all()

    def value(self):
        with self._condition:
            while not self._called:
                self._condition.wait()
            return self._value


class _Handler:
    def __init__(self, control, thread_pool):
        self._control = control
        self._thread_pool = thread_pool
        non_blocking_functions = (
            self.handle_unary_stream_non_blocking,
            self.handle_stream_stream_non_blocking,
        )
        for non_blocking_function in non_blocking_functions:
            non_blocking_function.__func__.experimental_non_blocking = True
            non_blocking_function.__func__.experimental_thread_pool = (
                self._thread_pool
            )

    def handle_unary_unary(self, request, servicer_context):
        self._control.control()
        if servicer_context is not None:
            servicer_context.set_trailing_metadata(
                (
                    (
                        "testkey",
                        "testvalue",
                    ),
                )
            )
            # TODO(https://github.com/grpc/grpc/issues/8483): test the values
            # returned by these methods rather than only "smoke" testing that
            # the return after having been called.
            servicer_context.is_active()
            servicer_context.time_remaining()
        return request

    def handle_unary_stream(self, request, servicer_context):
        for _ in range(test_constants.STREAM_LENGTH):
            self._control.control()
            yield request
        self._control.control()
        if servicer_context is not None:
            servicer_context.set_trailing_metadata(
                (
                    (
                        "testkey",
                        "testvalue",
                    ),
                )
            )

    def handle_unary_stream_non_blocking(
        self, request, servicer_context, on_next
    ):
        for _ in range(test_constants.STREAM_LENGTH):
            self._control.control()
            on_next(request)
        self._control.control()
        if servicer_context is not None:
            servicer_context.set_trailing_metadata(
                (
                    (
                        "testkey",
                        "testvalue",
                    ),
                )
            )
        on_next(None)

    def handle_stream_unary(self, request_iterator, servicer_context):
        if servicer_context is not None:
            servicer_context.invocation_metadata()
        self._control.control()
        response_elements = []
        for request in request_iterator:
            self._control.control()
            response_elements.append(request)
        self._control.control()
        if servicer_context is not None:
            servicer_context.set_trailing_metadata(
                (
                    (
                        "testkey",
                        "testvalue",
                    ),
                )
            )
        return b"".join(response_elements)

    def handle_stream_stream(self, request_iterator, servicer_context):
        self._control.control()
        if servicer_context is not None:
            servicer_context.set_trailing_metadata(
                (
                    (
                        "testkey",
                        "testvalue",
                    ),
                )
            )
        for request in request_iterator:
            self._control.control()
            yield request
        self._control.control()

    def handle_stream_stream_non_blocking(
        self, request_iterator, servicer_context, on_next
    ):
        self._control.control()
        if servicer_context is not None:
            servicer_context.set_trailing_metadata(
                (
                    (
                        "testkey",
                        "testvalue",
                    ),
                )
            )
        for request in request_iterator:
            self._control.control()
            on_next(request)
        self._control.control()
        on_next(None)


class _MethodHandler(grpc.RpcMethodHandler):
    def __init__(
        self,
        request_streaming,
        response_streaming,
        request_deserializer,
        response_serializer,
        unary_unary,
        unary_stream,
        stream_unary,
        stream_stream,
    ):
        self.request_streaming = request_streaming
        self.response_streaming = response_streaming
        self.request_deserializer = request_deserializer
        self.response_serializer = response_serializer
        self.unary_unary = unary_unary
        self.unary_stream = unary_stream
        self.stream_unary = stream_unary
        self.stream_stream = stream_stream


class _GenericHandler(grpc.GenericRpcHandler):
    def __init__(self, handler):
        self._handler = handler

    def service(self, handler_call_details):
        if handler_call_details.method == _UNARY_UNARY:
            return _MethodHandler(
                False,
                False,
                None,
                None,
                self._handler.handle_unary_unary,
                None,
                None,
                None,
            )
        elif handler_call_details.method == _UNARY_STREAM:
            return _MethodHandler(
                False,
                True,
                _DESERIALIZE_REQUEST,
                _SERIALIZE_RESPONSE,
                None,
                self._handler.handle_unary_stream,
                None,
                None,
            )
        elif handler_call_details.method == _UNARY_STREAM_NON_BLOCKING:
            return _MethodHandler(
                False,
                True,
                _DESERIALIZE_REQUEST,
                _SERIALIZE_RESPONSE,
                None,
                self._handler.handle_unary_stream_non_blocking,
                None,
                None,
            )
        elif handler_call_details.method == _STREAM_UNARY:
            return _MethodHandler(
                True,
                False,
                _DESERIALIZE_REQUEST,
                _SERIALIZE_RESPONSE,
                None,
                None,
                self._handler.handle_stream_unary,
                None,
            )
        elif handler_call_details.method == _STREAM_STREAM:
            return _MethodHandler(
                True,
                True,
                None,
                None,
                None,
                None,
                None,
                self._handler.handle_stream_stream,
            )
        elif handler_call_details.method == _STREAM_STREAM_NON_BLOCKING:
            return _MethodHandler(
                True,
                True,
                None,
                None,
                None,
                None,
                None,
                self._handler.handle_stream_stream_non_blocking,
            )
        else:
            return None


def get_method_handlers(handler):
    return {
        _UNARY_UNARY: _MethodHandler(
            False,
            False,
            None,
            None,
            handler.handle_unary_unary,
            None,
            None,
            None,
        ),
        _UNARY_STREAM: _MethodHandler(
            False,
            True,
            _DESERIALIZE_REQUEST,
            _SERIALIZE_RESPONSE,
            None,
            handler.handle_unary_stream,
            None,
            None,
        ),
        _UNARY_STREAM_NON_BLOCKING: _MethodHandler(
            False,
            True,
            _DESERIALIZE_REQUEST,
            _SERIALIZE_RESPONSE,
            None,
            handler.handle_unary_stream_non_blocking,
            None,
            None,
        ),
        _STREAM_UNARY: _MethodHandler(
            True,
            False,
            _DESERIALIZE_REQUEST,
            _SERIALIZE_RESPONSE,
            None,
            None,
            handler.handle_stream_unary,
            None,
        ),
        _STREAM_STREAM: _MethodHandler(
            True,
            True,
            None,
            None,
            None,
            None,
            None,
            handler.handle_stream_stream,
        ),
        _STREAM_STREAM_NON_BLOCKING: _MethodHandler(
            True,
            True,
            None,
            None,
            None,
            None,
            None,
            handler.handle_stream_stream_non_blocking,
        ),
    }


def unary_unary_multi_callable(channel):
    return channel.unary_unary(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_UNARY),
        _registered_method=True,
    )


def unary_stream_multi_callable(channel):
    return channel.unary_stream(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_STREAM),
        request_serializer=_SERIALIZE_REQUEST,
        response_deserializer=_DESERIALIZE_RESPONSE,
        _registered_method=True,
    )


def unary_stream_non_blocking_multi_callable(channel):
    return channel.unary_stream(
        grpc._common.fully_qualified_method(
            _SERVICE_NAME, _UNARY_STREAM_NON_BLOCKING
        ),
        request_serializer=_SERIALIZE_REQUEST,
        response_deserializer=_DESERIALIZE_RESPONSE,
        _registered_method=True,
    )


def stream_unary_multi_callable(channel):
    return channel.stream_unary(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_UNARY),
        request_serializer=_SERIALIZE_REQUEST,
        response_deserializer=_DESERIALIZE_RESPONSE,
        _registered_method=True,
    )


def stream_stream_multi_callable(channel):
    return channel.stream_stream(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_STREAM),
        _registered_method=True,
    )


def stream_stream_non_blocking_multi_callable(channel):
    return channel.stream_stream(
        grpc._common.fully_qualified_method(
            _SERVICE_NAME, _STREAM_STREAM_NON_BLOCKING
        ),
        _registered_method=True,
    )


class BaseRPCTest:
    def setUp(self):
        self._control = test_control.PauseFailControl()
        self._thread_pool = thread_pool.RecordingThreadPool(max_workers=None)
        self._handler = _Handler(self._control, self._thread_pool)

        self._server = test_common.test_server()
        port = self._server.add_insecure_port("[::]:0")
        self._server.add_registered_method_handlers(
            _SERVICE_NAME, get_method_handlers(self._handler)
        )
        self._server.start()

        self._channel = grpc.insecure_channel("localhost:%d" % port)

    def tearDown(self):
        self._server.stop(None)
        self._channel.close()

    def _consume_one_stream_response_unary_request(self, multi_callable):
        request = b"\x57\x38"

        response_iterator = multi_callable(
            request,
            metadata=(("test", "ConsumingOneStreamResponseUnaryRequest"),),
        )
        next(response_iterator)

    def _consume_some_but_not_all_stream_responses_unary_request(
        self, multi_callable
    ):
        request = b"\x57\x38"

        response_iterator = multi_callable(
            request,
            metadata=(
                ("test", "ConsumingSomeButNotAllStreamResponsesUnaryRequest"),
            ),
        )
        for _ in range(test_constants.STREAM_LENGTH // 2):
            next(response_iterator)

    def _consume_some_but_not_all_stream_responses_stream_request(
        self, multi_callable
    ):
        requests = tuple(
            b"\x67\x88" for _ in range(test_constants.STREAM_LENGTH)
        )
        request_iterator = iter(requests)

        response_iterator = multi_callable(
            request_iterator,
            metadata=(
                ("test", "ConsumingSomeButNotAllStreamResponsesStreamRequest"),
            ),
        )
        for _ in range(test_constants.STREAM_LENGTH // 2):
            next(response_iterator)

    def _consume_too_many_stream_responses_stream_request(self, multi_callable):
        requests = tuple(
            b"\x67\x88" for _ in range(test_constants.STREAM_LENGTH)
        )
        request_iterator = iter(requests)

        response_iterator = multi_callable(
            request_iterator,
            metadata=(
                ("test", "ConsumingTooManyStreamResponsesStreamRequest"),
            ),
        )
        for _ in range(test_constants.STREAM_LENGTH):
            next(response_iterator)
        for _ in range(test_constants.STREAM_LENGTH):
            with self.assertRaises(StopIteration):
                next(response_iterator)

        self.assertIsNotNone(response_iterator.initial_metadata())
        self.assertIs(grpc.StatusCode.OK, response_iterator.code())
        self.assertIsNotNone(response_iterator.details())
        self.assertIsNotNone(response_iterator.trailing_metadata())

    def _cancelled_unary_request_stream_response(self, multi_callable):
        request = b"\x07\x19"

        with self._control.pause():
            response_iterator = multi_callable(
                request,
                metadata=(("test", "CancelledUnaryRequestStreamResponse"),),
            )
            self._control.block_until_paused()
            response_iterator.cancel()

        with self.assertRaises(grpc.RpcError) as exception_context:
            next(response_iterator)
        self.assertIs(
            grpc.StatusCode.CANCELLED, exception_context.exception.code()
        )
        self.assertIsNotNone(response_iterator.initial_metadata())
        self.assertIs(grpc.StatusCode.CANCELLED, response_iterator.code())
        self.assertIsNotNone(response_iterator.details())
        self.assertIsNotNone(response_iterator.trailing_metadata())

    def _cancelled_stream_request_stream_response(self, multi_callable):
        requests = tuple(
            b"\x07\x08" for _ in range(test_constants.STREAM_LENGTH)
        )
        request_iterator = iter(requests)

        with self._control.pause():
            response_iterator = multi_callable(
                request_iterator,
                metadata=(("test", "CancelledStreamRequestStreamResponse"),),
            )
            response_iterator.cancel()

        with self.assertRaises(grpc.RpcError):
            next(response_iterator)
        self.assertIsNotNone(response_iterator.initial_metadata())
        self.assertIs(grpc.StatusCode.CANCELLED, response_iterator.code())
        self.assertIsNotNone(response_iterator.details())
        self.assertIsNotNone(response_iterator.trailing_metadata())

    def _expired_unary_request_stream_response(self, multi_callable):
        request = b"\x07\x19"

        with self._control.pause():
            with self.assertRaises(grpc.RpcError) as exception_context:
                response_iterator = multi_callable(
                    request,
                    timeout=test_constants.SHORT_TIMEOUT,
                    metadata=(("test", "ExpiredUnaryRequestStreamResponse"),),
                )
                next(response_iterator)

        self.assertIs(
            grpc.StatusCode.DEADLINE_EXCEEDED,
            exception_context.exception.code(),
        )
        self.assertIs(
            grpc.StatusCode.DEADLINE_EXCEEDED, response_iterator.code()
        )

    def _expired_stream_request_stream_response(self, multi_callable):
        requests = tuple(
            b"\x67\x18" for _ in range(test_constants.STREAM_LENGTH)
        )
        request_iterator = iter(requests)

        with self._control.pause():
            with self.assertRaises(grpc.RpcError) as exception_context:
                response_iterator = multi_callable(
                    request_iterator,
                    timeout=test_constants.SHORT_TIMEOUT,
                    metadata=(("test", "ExpiredStreamRequestStreamResponse"),),
                )
                next(response_iterator)

        self.assertIs(
            grpc.StatusCode.DEADLINE_EXCEEDED,
            exception_context.exception.code(),
        )
        self.assertIs(
            grpc.StatusCode.DEADLINE_EXCEEDED, response_iterator.code()
        )

    def _failed_unary_request_stream_response(self, multi_callable):
        request = b"\x37\x17"

        with self.assertRaises(grpc.RpcError) as exception_context:
            with self._control.fail():
                response_iterator = multi_callable(
                    request,
                    metadata=(("test", "FailedUnaryRequestStreamResponse"),),
                )
                next(response_iterator)

        self.assertIs(
            grpc.StatusCode.UNKNOWN, exception_context.exception.code()
        )

    def _failed_stream_request_stream_response(self, multi_callable):
        requests = tuple(
            b"\x67\x88" for _ in range(test_constants.STREAM_LENGTH)
        )
        request_iterator = iter(requests)

        with self._control.fail():
            with self.assertRaises(grpc.RpcError) as exception_context:
                response_iterator = multi_callable(
                    request_iterator,
                    metadata=(("test", "FailedStreamRequestStreamResponse"),),
                )
                tuple(response_iterator)

        self.assertIs(
            grpc.StatusCode.UNKNOWN, exception_context.exception.code()
        )
        self.assertIs(grpc.StatusCode.UNKNOWN, response_iterator.code())

    def _ignored_unary_stream_request_future_unary_response(
        self, multi_callable
    ):
        request = b"\x37\x17"

        multi_callable(
            request, metadata=(("test", "IgnoredUnaryRequestStreamResponse"),)
        )

    def _ignored_stream_request_stream_response(self, multi_callable):
        requests = tuple(
            b"\x67\x88" for _ in range(test_constants.STREAM_LENGTH)
        )
        request_iterator = iter(requests)

        multi_callable(
            request_iterator,
            metadata=(("test", "IgnoredStreamRequestStreamResponse"),),
        )



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests AsyncIO version of grpcio-health-checking."""

import asyncio
import logging
import random
import time
import unittest

import grpc
from grpc.experimental import aio
from grpc_health.v1 import health
from grpc_health.v1 import health_pb2
from grpc_health.v1 import health_pb2_grpc

from tests.unit.framework.common import test_constants
from tests_aio.unit._test_base import AioTestBase

_SERVING_SERVICE = "grpc.test.TestServiceServing"
_UNKNOWN_SERVICE = "grpc.test.TestServiceUnknown"
_NOT_SERVING_SERVICE = "grpc.test.TestServiceNotServing"
_WATCH_SERVICE = "grpc.test.WatchService"

_LARGE_NUMBER_OF_STATUS_CHANGES = 1000


async def _pipe_to_queue(call, queue):
    async for response in call:
        await queue.put(response)


class HealthServicerTest(AioTestBase):
    async def setUp(self):
        self._servicer = health.aio.HealthServicer()
        await self._servicer.set(
            _SERVING_SERVICE, health_pb2.HealthCheckResponse.SERVING
        )
        await self._servicer.set(
            _UNKNOWN_SERVICE, health_pb2.HealthCheckResponse.UNKNOWN
        )
        await self._servicer.set(
            _NOT_SERVING_SERVICE, health_pb2.HealthCheckResponse.NOT_SERVING
        )
        self._server = aio.server()
        port = self._server.add_insecure_port("[::]:0")
        health_pb2_grpc.add_HealthServicer_to_server(
            self._servicer, self._server
        )
        await self._server.start()

        self._channel = aio.insecure_channel("localhost:%d" % port)
        self._stub = health_pb2_grpc.HealthStub(self._channel)

    async def tearDown(self):
        await self._channel.close()
        await self._server.stop(None)

    async def test_check_empty_service(self):
        request = health_pb2.HealthCheckRequest()
        resp = await self._stub.Check(request)
        self.assertEqual(health_pb2.HealthCheckResponse.SERVING, resp.status)

    async def test_check_serving_service(self):
        request = health_pb2.HealthCheckRequest(service=_SERVING_SERVICE)
        resp = await self._stub.Check(request)
        self.assertEqual(health_pb2.HealthCheckResponse.SERVING, resp.status)

    async def test_check_unknown_service(self):
        request = health_pb2.HealthCheckRequest(service=_UNKNOWN_SERVICE)
        resp = await self._stub.Check(request)
        self.assertEqual(health_pb2.HealthCheckResponse.UNKNOWN, resp.status)

    async def test_check_not_serving_service(self):
        request = health_pb2.HealthCheckRequest(service=_NOT_SERVING_SERVICE)
        resp = await self._stub.Check(request)
        self.assertEqual(
            health_pb2.HealthCheckResponse.NOT_SERVING, resp.status
        )

    async def test_check_not_found_service(self):
        request = health_pb2.HealthCheckRequest(service="not-found")
        with self.assertRaises(aio.AioRpcError) as context:
            await self._stub.Check(request)

        self.assertEqual(grpc.StatusCode.NOT_FOUND, context.exception.code())

    async def test_health_service_name(self):
        self.assertEqual(health.SERVICE_NAME, "grpc.health.v1.Health")

    async def test_watch_empty_service(self):
        request = health_pb2.HealthCheckRequest(service=health.OVERALL_HEALTH)

        call = self._stub.Watch(request)
        queue = asyncio.Queue()
        task = self.loop.create_task(_pipe_to_queue(call, queue))

        self.assertEqual(
            health_pb2.HealthCheckResponse.SERVING, (await queue.get()).status
        )

        call.cancel()

        with self.assertRaises(asyncio.CancelledError):
            await task

        self.assertTrue(queue.empty())

    async def test_watch_new_service(self):
        request = health_pb2.HealthCheckRequest(service=_WATCH_SERVICE)
        call = self._stub.Watch(request)
        queue = asyncio.Queue()
        task = self.loop.create_task(_pipe_to_queue(call, queue))

        self.assertEqual(
            health_pb2.HealthCheckResponse.SERVICE_UNKNOWN,
            (await queue.get()).status,
        )

        await self._servicer.set(
            _WATCH_SERVICE, health_pb2.HealthCheckResponse.SERVING
        )
        self.assertEqual(
            health_pb2.HealthCheckResponse.SERVING, (await queue.get()).status
        )

        await self._servicer.set(
            _WATCH_SERVICE, health_pb2.HealthCheckResponse.NOT_SERVING
        )
        self.assertEqual(
            health_pb2.HealthCheckResponse.NOT_SERVING,
            (await queue.get()).status,
        )

        call.cancel()

        with self.assertRaises(asyncio.CancelledError):
            await task

        self.assertTrue(queue.empty())

    async def test_watch_service_isolation(self):
        request = health_pb2.HealthCheckRequest(service=_WATCH_SERVICE)
        call = self._stub.Watch(request)
        queue = asyncio.Queue()
        task = self.loop.create_task(_pipe_to_queue(call, queue))

        self.assertEqual(
            health_pb2.HealthCheckResponse.SERVICE_UNKNOWN,
            (await queue.get()).status,
        )

        await self._servicer.set(
            "some-other-service", health_pb2.HealthCheckResponse.SERVING
        )
        # The change of health status in other service should be isolated.
        # Hence, no additional notification should be observed.
        with self.assertRaises(asyncio.TimeoutError):
            await asyncio.wait_for(queue.get(), test_constants.SHORT_TIMEOUT)

        call.cancel()

        with self.assertRaises(asyncio.CancelledError):
            await task

        self.assertTrue(queue.empty())

    async def test_two_watchers(self):
        request = health_pb2.HealthCheckRequest(service=_WATCH_SERVICE)
        queue1 = asyncio.Queue()
        queue2 = asyncio.Queue()
        call1 = self._stub.Watch(request)
        call2 = self._stub.Watch(request)
        task1 = self.loop.create_task(_pipe_to_queue(call1, queue1))
        task2 = self.loop.create_task(_pipe_to_queue(call2, queue2))

        self.assertEqual(
            health_pb2.HealthCheckResponse.SERVICE_UNKNOWN,
            (await queue1.get()).status,
        )
        self.assertEqual(
            health_pb2.HealthCheckResponse.SERVICE_UNKNOWN,
            (await queue2.get()).status,
        )

        await self._servicer.set(
            _WATCH_SERVICE, health_pb2.HealthCheckResponse.SERVING
        )
        self.assertEqual(
            health_pb2.HealthCheckResponse.SERVING, (await queue1.get()).status
        )
        self.assertEqual(
            health_pb2.HealthCheckResponse.SERVING, (await queue2.get()).status
        )

        call1.cancel()
        call2.cancel()

        with self.assertRaises(asyncio.CancelledError):
            await task1

        with self.assertRaises(asyncio.CancelledError):
            await task2

        self.assertTrue(queue1.empty())
        self.assertTrue(queue2.empty())

    async def test_cancelled_watch_removed_from_watch_list(self):
        request = health_pb2.HealthCheckRequest(service=_WATCH_SERVICE)
        call = self._stub.Watch(request)
        queue = asyncio.Queue()
        task = self.loop.create_task(_pipe_to_queue(call, queue))

        self.assertEqual(
            health_pb2.HealthCheckResponse.SERVICE_UNKNOWN,
            (await queue.get()).status,
        )

        call.cancel()
        await self._servicer.set(
            _WATCH_SERVICE, health_pb2.HealthCheckResponse.SERVING
        )

        with self.assertRaises(asyncio.CancelledError):
            await task

        # Wait for the serving coroutine to process client cancellation.
        timeout = time.monotonic() + test_constants.TIME_ALLOWANCE
        while time.monotonic() < timeout and self._servicer._server_watchers:
            await asyncio.sleep(1)
        self.assertFalse(
            self._servicer._server_watchers,
            "There should not be any watcher left",
        )
        self.assertTrue(queue.empty())

    async def test_graceful_shutdown(self):
        request = health_pb2.HealthCheckRequest(service=health.OVERALL_HEALTH)
        call = self._stub.Watch(request)
        queue = asyncio.Queue()
        task = self.loop.create_task(_pipe_to_queue(call, queue))

        self.assertEqual(
            health_pb2.HealthCheckResponse.SERVING, (await queue.get()).status
        )

        await self._servicer.enter_graceful_shutdown()
        self.assertEqual(
            health_pb2.HealthCheckResponse.NOT_SERVING,
            (await queue.get()).status,
        )

        # This should be a no-op.
        await self._servicer.set(
            health.OVERALL_HEALTH, health_pb2.HealthCheckResponse.SERVING
        )

        resp = await self._stub.Check(request)
        self.assertEqual(
            health_pb2.HealthCheckResponse.NOT_SERVING, resp.status
        )

        call.cancel()

        with self.assertRaises(asyncio.CancelledError):
            await task

        self.assertTrue(queue.empty())

    async def test_no_duplicate_status(self):
        request = health_pb2.HealthCheckRequest(service=_WATCH_SERVICE)
        call = self._stub.Watch(request)
        queue = asyncio.Queue()
        task = self.loop.create_task(_pipe_to_queue(call, queue))

        self.assertEqual(
            health_pb2.HealthCheckResponse.SERVICE_UNKNOWN,
            (await queue.get()).status,
        )
        last_status = health_pb2.HealthCheckResponse.SERVICE_UNKNOWN

        for _ in range(_LARGE_NUMBER_OF_STATUS_CHANGES):
            if random.randint(0, 1) == 0:
                status = health_pb2.HealthCheckResponse.SERVING
            else:
                status = health_pb2.HealthCheckResponse.NOT_SERVING

            await self._servicer.set(_WATCH_SERVICE, status)
            if status != last_status:
                self.assertEqual(status, (await queue.get()).status)
            last_status = status

        call.cancel()

        with self.assertRaises(asyncio.CancelledError):
            await task

        self.assertTrue(queue.empty())


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# Copyright 2021 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from concurrent import futures
from typing import Any, Tuple

import gevent
import grpc

from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc

LONG_UNARY_CALL_WITH_SLEEP_VALUE = 1


class TestServiceServicer(test_pb2_grpc.TestServiceServicer):
    def UnaryCall(self, request, context):
        return messages_pb2.SimpleResponse()

    def UnaryCallWithSleep(self, unused_request, unused_context):
        gevent.sleep(LONG_UNARY_CALL_WITH_SLEEP_VALUE)
        return messages_pb2.SimpleResponse()


def start_test_server(port: int = 0) -> Tuple[str, Any]:
    server = grpc.server(futures.ThreadPoolExecutor())
    servicer = TestServiceServicer()
    test_pb2_grpc.add_TestServiceServicer_to_server(
        TestServiceServicer(), server
    )

    server.add_generic_rpc_handlers((_create_extra_generic_handler(servicer),))
    port = server.add_insecure_port("[::]:%d" % port)
    server.start()
    return "localhost:%d" % port, server


def _create_extra_generic_handler(servicer: TestServiceServicer) -> Any:
    # Add programmatically extra methods not provided by the proto file
    # that are used during the tests
    rpc_method_handlers = {
        "UnaryCallWithSleep": grpc.unary_unary_rpc_method_handler(
            servicer.UnaryCallWithSleep,
            request_deserializer=messages_pb2.SimpleRequest.FromString,
            response_serializer=messages_pb2.SimpleResponse.SerializeToString,
        )
    }
    return grpc.method_handlers_generic_handler(
        "grpc.testing.TestService", rpc_method_handlers
    )



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Buildgen expand binary attributes plugin.

This fills in any optional attributes.

"""


def mako_plugin(dictionary):
    """The exported plugin code for expand_filegroups.

    The list of libs in the build.yaml file can contain "filegroups" tags.
    These refer to the filegroups in the root object. We will expand and
    merge filegroups on the src, headers and public_headers properties.

    """

    targets = dictionary.get("targets")
    default_platforms = ["windows", "posix", "linux", "mac"]

    for tgt in targets:
        tgt["flaky"] = tgt.get("flaky", False)
        tgt["platforms"] = sorted(tgt.get("platforms", default_platforms))
        tgt["ci_platforms"] = sorted(tgt.get("ci_platforms", tgt["platforms"]))
        tgt["boringssl"] = tgt.get("boringssl", False)
        tgt["zlib"] = tgt.get("zlib", False)
        tgt["ares"] = tgt.get("ares", False)
        tgt["gtest"] = tgt.get("gtest", False)

    libs = dictionary.get("libs")
    for lib in libs:
        lib["boringssl"] = lib.get("boringssl", False)
        lib["zlib"] = lib.get("zlib", False)
        lib["ares"] = lib.get("ares", False)



# Copyright 2019 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
import functools
import logging
from typing import Callable
import unittest

from grpc.experimental import aio

__all__ = "AioTestBase"

_COROUTINE_FUNCTION_ALLOWLIST = ["setUp", "tearDown"]


def _async_to_sync_decorator(f: Callable, loop: asyncio.AbstractEventLoop):
    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        return loop.run_until_complete(f(*args, **kwargs))

    return wrapper


def _get_default_loop(debug=True):
    loop = None
    try:
        loop = asyncio.get_event_loop()
    except:
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
    finally:
        if loop:
            loop.set_debug(debug)
    return loop


# NOTE(gnossen) this test class can also be implemented with metaclass.
class AioTestBase(unittest.TestCase):
    # NOTE(lidi) We need to pick a loop for entire testing phase, otherwise it
    # will trigger create new loops in new threads, leads to deadlock.
    _TEST_LOOP = _get_default_loop()

    @property
    def loop(self):
        return self._TEST_LOOP

    def __getattribute__(self, name):
        """Overrides the loading logic to support coroutine functions."""
        attr = super().__getattribute__(name)

        # If possible, converts the coroutine into a sync function.
        if name.startswith("test_") or name in _COROUTINE_FUNCTION_ALLOWLIST:
            if asyncio.iscoroutinefunction(attr):
                return _async_to_sync_decorator(attr, self._TEST_LOOP)
        # For other attributes, let them pass.
        return attr
